//-------------------------------------------------------------------------------
// Tutorial 1 de Direct3D 9
// PLANET ISO
// http://www.lessman.net/planetiso
//-------------------------------------------------------------------------------

#include <windows.h>
#include <d3d9.h>

//-------------------------------------------------------------------------------
// Definiciones
//-------------------------------------------------------------------------------
#define APP_NAME	"D3D 1 (DirectX 9)"
#define WND_WIDTH	640
#define WND_HEIGHT	480

// Nos dice si una tecla determinada ha sido pulsada
#define GET_KEYDOWN(key) ((int)((GetAsyncKeyState(key) & (1 << 15)) >> 15))

//-------------------------------------------------------------------------------
// Declaracin de funciones
//-------------------------------------------------------------------------------
LRESULT WINAPI	MsgProc(HWND, UINT, WPARAM, LPARAM);
INT WINAPI		WinMain(HINSTANCE, HINSTANCE, LPSTR, int);

HRESULT			IniVentana(HINSTANCE hInstance);

HRESULT			InicializarD3D(HWND hWnd);
void			FinalizarD3D(void);
void			Render(void);

//-----------------------------------------------------------------------------
// Declaracin de variabes globales
//-----------------------------------------------------------------------------
HWND				g_hWnd;				// Nuestra ventana
LPDIRECT3D9			g_pD3D = NULL;		// Objeto D3D usado para crear el dispositivo D3D
LPDIRECT3DDEVICE9	g_pD3DDev = NULL;	// Dispositivo D3D
BOOL				g_bWindowed = TRUE;	// A pantalla completa o en ventana? (por defecto ventana)

//----------------------------------------------------------------------------
// Funcin: IniVentana
// Propsito: Inicializa nuestra ventana
//----------------------------------------------------------------------------
HRESULT IniVentana(HINSTANCE hInstance)
{
	
	WNDCLASSEX	wc;		// clase de ventana
	
	// Rellenamos la estructura de clase de ventana
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_CLASSDC;
	wc.lpfnWndProc = MsgProc;
	wc.cbClsExtra = 0L;
	wc.cbWndExtra = 0L;
	wc.hInstance = GetModuleHandle(NULL);
	wc.hIcon = NULL;
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = APP_NAME;
	wc.hIconSm = NULL;


	// Registramos la clase de vetana
	if(!RegisterClassEx(&wc))
	{
		MessageBox(NULL, "No se pudo registrar la clase de ventana", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	// Creamos la ventana
	g_hWnd = CreateWindow(APP_NAME,				// nombre de la clase
						APP_NAME,				// ttulo de la ventana
						WS_OVERLAPPEDWINDOW,	// tipo de ventana
						CW_USEDEFAULT,			// posicin X de la ventana
						CW_USEDEFAULT,			// posicin Y de a ventana
						WND_WIDTH,				// ancho de la ventana
						WND_HEIGHT,				// alto de la ventana
						GetDesktopWindow(),		// ventana padre
						NULL,					// men
						wc.hInstance,			// instancia de la aplicacin
						NULL);					// no se suele usar

	if (!g_hWnd)
    {
        MessageBox(NULL,"No se cre el objeto g_hWnd","Error",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
    }

	return S_OK;
}

//----------------------------------------------------------------------------
// Funcin: InicializarD3D
// Propsito: Inicia el objeto D3D y el dispositivo D3D
//----------------------------------------------------------------------------
HRESULT InicializarD3D(HWND hWnd)
{
	// En primer lugar creamos el objeto D3D
	if((g_pD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
		return E_FAIL;

	// Tomamos las propiedades de nuestro escritorio (profundidad de color, resolucin, etc)
	D3DDISPLAYMODE d3ddm;
	// Especificamos como dispositivo de visualizacin el dispositivo primario
	if(FAILED(g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
		return E_FAIL;

	// Ahora rellenamos la estructura usada para crear el dispositivo
	// Indicamos que queremos un backbuffer.
	// Ajustamos la anchura y la altura del backbuffer al tamao de nuestra
	// ventana.
	// Especificamos una aplicacin a pantalla completa.
	// Especificamos que D3D escoja el sistema ms ptimo para realizar
	// el intercambio del backbuffer a frontbuffer.
	// Especificamos un formato para el backbuffer igual que el de nuestro
	// escritorio.
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp, sizeof(d3dpp));
	d3dpp.BackBufferCount = 1;
	d3dpp.BackBufferWidth = WND_WIDTH;
	d3dpp.BackBufferHeight = WND_HEIGHT;
	d3dpp.Windowed = g_bWindowed;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = d3ddm.Format;

	// Por ltimo creamos el dispositivo D3D (la aceleradora 3D propiamente dicha)
	// Escogemos el dispositivo por defecto (el primario)
	// Especificamos que queremos un dispositivo HAL, es decir, que tenga aceleracin
	// a traves de D3D.
	// Especificamos la ventana que se asociar al dispositivo.
	// Especificamos que queremos que D3D procese los vrtices por software...esto
	// se debera comprobar ya que si disponemos de un dispositivo TnL tal como
	// GeForce, preferiremos que se realice por hardware.
	if(FAILED(g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
									D3DCREATE_SOFTWARE_VERTEXPROCESSING,
									&d3dpp, &g_pD3DDev)))
	{
		MessageBox(g_hWnd, "No se pudo crear el dispositivo 3D", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	return S_OK;
}

//----------------------------------------------------------------------------
// Funcin: FinalizarD3D
// Propsito: Elimina el dispositivo D3D y el objeto D3D...por ese orden!
//----------------------------------------------------------------------------
void FinalizarD3D(void)
{
	if(g_pD3DDev != NULL)
		g_pD3DDev->Release();

	if(g_pD3D != NULL)
		g_pD3D->Release();
}

//----------------------------------------------------------------------------
// Funcin: Render
// Propsito: Aqu se pondrn nuestras funciones de render
//----------------------------------------------------------------------------
void Render(void)
{
	if(NULL == g_pD3DDev)
		return;

	// Limpia la pantalla de un color (naranja en este caso)
	g_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(255,127,0), 1.0f, 0);
    
	// Indica a D3D que en esta seccin pondremos nuestras 
	// funciones de render
	g_pD3DDev->BeginScene();


    //---------------------------------------//
    // Funciones de render aqu
	//---------------------------------------//

    
	// A partir de aqu ya no podemos poner ms funciones
	// de render
	g_pD3DDev->EndScene();
    
	// Finalmente se pasa el contenido 
	// del backbuffer al dispositivo.
	g_pD3DDev->Present(NULL, NULL, g_hWnd, NULL);
}

//-------------------------------------------------------------------------------
// Funcin: MsgProc
// Propsito: Manejar los mensajes de la aplicacin
//-------------------------------------------------------------------------------
LRESULT WINAPI MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_DESTROY:
		
			PostQuitMessage(0);
			return 0;

		case WM_PAINT:

			ValidateRect(hWnd, NULL);
			return 0;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

//----------------------------------------------------------------------------
// Funcin: WinMain
// Propsito; Es la funcin principal de toda aplicacin de Windows
// Se encarga de crear la ventana, mostrarla y es la receptora de los
// mensajes que le son enviados a la aplicacin
//----------------------------------------------------------------------------
INT WINAPI WinMain(HINSTANCE hInstance,			// Instancia a nuestra aplicacin
				   HINSTANCE hPrevInstance,		// Existen instancias previas de nuestra aplicacin?
				   LPSTR lpCmdLine,				// Lnea de comandos
				   int nShowCmd)				// Cmo se despliega nuestra ventana?
{
	
	if(!SUCCEEDED(IniVentana(hInstance)))
		return 0;

	if(SUCCEEDED(InicializarD3D(g_hWnd)))
	{
		// Mostrar nuestra ventana
		ShowWindow(g_hWnd, nShowCmd);
		UpdateWindow(g_hWnd);

		// Si est en modo ventana mostramos el cursor
		if(g_bWindowed)
			ShowCursor(TRUE);
		else	// de lo contrario lo ocultamos
			ShowCursor(FALSE);

		// Bucle de mensajes tpico de Windows
		MSG	msg;	// handle a un mensaje
		ZeroMemory(&msg, sizeof(msg));

		while(msg.message != WM_QUIT)
		{
			// si se pulsa ESC salimos
			if(GET_KEYDOWN(VK_ESCAPE))
				PostQuitMessage(0);

			if(PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			else
			{
				Render();
			}
		}
	}

    // Salir de la aplicacin
	FinalizarD3D();
	UnregisterClass(APP_NAME,hInstance);

	return (0);
}