//-------------------------------------------------------------------------------
// Tutorial 3 de Direct3D 9
// GEARDOME ENTERTAINMENT
// http://www.geardome.com
//-------------------------------------------------------------------------------

#include <windows.h>
#include <d3dx9.h>

//-------------------------------------------------------------------------------
// Definiciones
//-------------------------------------------------------------------------------

#define APP_NAME    "D3D 3"
#define WND_WIDTH	640
#define WND_HEIGHT	480

// Nos dice si una tecla determinada ha sido pulsada
#define GET_KEYDOWN(key) ((int)((GetAsyncKeyState(key) & (1 << 15)) >> 15))


// Coordenadas XYZ y el color del vrtice
#define D3DFVF_MIVERTICE (D3DFVF_XYZ|D3DFVF_DIFFUSE)

// Nuestro tipo de vrtice
struct MIVERTICE
{
	float x, y, z;		// posicin de un vrtice
	DWORD color;		// color del vrtice
};

//-------------------------------------------------------------------------------
// Declaracin de funciones
//-------------------------------------------------------------------------------
LRESULT WINAPI	MsgProc(HWND, UINT, WPARAM, LPARAM);
INT WINAPI		WinMain(HINSTANCE, HINSTANCE, LPSTR, int);

HRESULT			IniVentana(HINSTANCE hInstance);

void			ActualizaMatrices(void);
HRESULT			IniGeometria(void);
HRESULT			InicializarD3D(HWND hWnd);
void			FinalizarD3D(void);
void			Render(void);



//-----------------------------------------------------------------------------
// Declaracin de variabes globales
//-----------------------------------------------------------------------------
HWND					g_hWnd;				// Nuestra ventana
LPDIRECT3D9				g_pD3D = NULL;		// Objeto D3D usado para crear el dispositivo D3D
LPDIRECT3DDEVICE9		g_pD3DDev = NULL;	// Dispositivo D3D
LPDIRECT3DVERTEXBUFFER9 g_pVB = NULL;		// Buffer que contendr los vrtices
BOOL					g_bWindowed = TRUE;	// A pantalla completa o en ventana? (por defecto ventana)

//----------------------------------------------------------------------------
// Funcin: IniVentana
// Propsito: Inicializa nuestra ventana
//----------------------------------------------------------------------------
HRESULT IniVentana(HINSTANCE hInstance)
{
	
	WNDCLASSEX	wc;		// clase de ventana
	
	// Rellenamos la estructura de clase de ventana
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_CLASSDC;
	wc.lpfnWndProc = MsgProc;
	wc.cbClsExtra = 0L;
	wc.cbWndExtra = 0L;
	wc.hInstance = GetModuleHandle(NULL);
	wc.hIcon = NULL;
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = APP_NAME;
	wc.hIconSm = NULL;


	// Registramos la clase de vetana
	if(!RegisterClassEx(&wc))
	{
		MessageBox(NULL, "No se pudo registrar la clase de ventana", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	// Creamos la ventana
	g_hWnd = CreateWindow(APP_NAME,				// nombre de la clase
						APP_NAME,				// ttulo de la ventana
						WS_OVERLAPPEDWINDOW,	// tipo de ventana
						CW_USEDEFAULT,			// posicin X de la ventana
						CW_USEDEFAULT,			// posicin Y de a ventana
						WND_WIDTH,				// ancho de la ventana
						WND_HEIGHT,				// alto de la ventana
						GetDesktopWindow(),		// ventana padre
						NULL,					// men
						wc.hInstance,			// instancia de la aplicacin
						NULL);					// no se suele usar

	if (!g_hWnd)
    {
        MessageBox(NULL,"No se cre el objeto g_hWnd","Error",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
    }

	return S_OK;
}

//----------------------------------------------------------------------------
// Funcin: InicializarD3D
// Propsito: Inicia el objeto D3D y el dispositivo D3D
//----------------------------------------------------------------------------
HRESULT InicializarD3D(HWND hWnd)
{
	// En primer lugar creamos el objeto D3D
	if((g_pD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
		return E_FAIL;

	// Tomamos las propiedades de nuestro escritorio (profundidad de color, resolucin, etc)
	D3DDISPLAYMODE d3ddm;
	// Especificamos como dispositivo de visualizacin el dispositivo primario
	if(FAILED(g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
		return E_FAIL;

	// Ahora rellenamos la estructura usada para crear el dispositivo
	// Indicamos que queremos un backbuffer.
	// Ajustamos la anchura y la altura del backbuffer al tamao de nuestra
	// ventana.
	// Especificamos una aplicacin a pantalla completa.
	// Especificamos que D3D escoja el sistema ms ptimo para realizar
	// el intercambio del backbuffer a frontbuffer.
	// Especificamos un formato para el backbuffer igual que el de nuestro
	// escritorio.
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp, sizeof(d3dpp));
	d3dpp.BackBufferCount = 1;
	d3dpp.BackBufferWidth = WND_WIDTH;
	d3dpp.BackBufferHeight = WND_HEIGHT;
	d3dpp.Windowed = g_bWindowed;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = d3ddm.Format;

	// Por ltimo creamos el dispositivo D3D (la aceleradora 3D propiamente dicha)
	// Escogemos el dispositivo por defecto (el primario)
	// Especificamos que queremos un dispositivo HAL, es decir, que tenga aceleracin
	// a traves de D3D.
	// Especificamos la ventana que se asociar al dispositivo.
	// Especificamos que queremos que D3D procese los vrtices por software...esto
	// se debera comprobar ya que si disponemos de un dispositivo TnL tal como
	// GeForce, preferiremos que se realice por hardware.
	if(FAILED(g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
									D3DCREATE_SOFTWARE_VERTEXPROCESSING,
									&d3dpp, &g_pD3DDev)))
	{
		MessageBox(g_hWnd, "No se pudo crear el dispositivo 3D", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	//Activa el back face culling. Lo hacemos porque no queremos que se pinte las
	//caras de los poligonos que no se ven
    g_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

	//Desactivamos la luz porque la estemos especificando en el color de vrtice
    g_pD3DDev->SetRenderState(D3DRS_LIGHTING, FALSE);

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: IniGeometria
// Propsito; Inicia el buffer de vrtices
//------------------------------------------------------------------------------
HRESULT	IniGeometria(void)
{
	// asignamos vrtices que forman un cubo con un color para cada vrtice
	// hay que asegurarse que los vertices se definen en el sentido de
	// las agujas del reloj ya que las caras contrarias las eliminamos con el culling
	
	MIVERTICE vertices[] =
    {	
		//Cara superior
		{-5.0f, 5.0f, -5.0f, D3DCOLOR_XRGB(0, 0, 255),},	//Vrtice 0 - Azul 
        {-5.0f, 5.0f, 5.0f, D3DCOLOR_XRGB(255, 0, 0),},		//Vrtice 1 - Rojo 
        {5.0f, 5.0f, -5.0f, D3DCOLOR_XRGB(255, 0, 0),},		//Vrtice 2 - Rojo 
		{5.0f, 5.0f, 5.0f, D3DCOLOR_XRGB(0, 255, 0),},		//Vrtice 3 - Verde 

		//Cara 1
		{-5.0f, -5.0f, -5.0f, D3DCOLOR_XRGB(255, 0, 0),},	//Vrtice 4 - Rojo 
        {-5.0f, 5.0f, -5.0f, D3DCOLOR_XRGB(0, 0, 255),},	//Vrtice 5 - Azul 
        {5.0f, -5.0f, -5.0f, D3DCOLOR_XRGB(0, 255, 0),},	//Vrtice 6 - Verde 
		{5.0f, 5.0f, -5.0f, D3DCOLOR_XRGB(255, 0, 0),},		//Vrtice 7 - Rojo 

		//Cara 2
		{5.0f, -5.0f, 5.0f, D3DCOLOR_XRGB(0, 0, 255),},		//Vrtice 8 - Azul 
        {5.0f, 5.0f, 5.0f, D3DCOLOR_XRGB(0, 255, 0),},		//Vrtice 9 - Verde
		
		//Cara 3
        {-5.0f, -5.0f, 5.0f, D3DCOLOR_XRGB(0, 255, 0),},	//Vrtice 10 - Verde 
		{-5.0f, 5.0f, 5.0f, D3DCOLOR_XRGB(255, 0, 0),},		//Vrtice 11 - Rojo 

		//Cara 4
        {-5.0f, -5.0f, -5.0f, D3DCOLOR_XRGB(255, 0, 0),},	//Vrtice 12 - Rojo 
        {-5.0f, 5.0f, -5.0f, D3DCOLOR_XRGB(0, 0, 255),},	//Vrtice 13 - Azul

		//Cara inferior
		{5.0f, -5.0f, -5.0f, D3DCOLOR_XRGB(0, 255, 0),},	//Vrtice 14 - Verde 
        {5.0f, -5.0f, 5.0f, D3DCOLOR_XRGB(0, 0, 255),},		//Vrtice 15 - Azul 
        {-5.0f, -5.0f, -5.0f, D3DCOLOR_XRGB(255, 0, 0),},	//Vrtice 16 - Rojo 
		{-5.0f, -5.0f, 5.0f, D3DCOLOR_XRGB(0, 255, 0),},	//Vrtice 17 - Verde
    };

	// Creamos el buffer de vrtices.
	// Primero especificamos el tamao...18 vrtices * el tamao de la estructura
	// que hemos definido.
	// Seguidamente especificamos el modo de uso.
	// El siguiente argumento es el tipo de vrtice.
	// Seguidamente especificamos que queremos que D3D elija la memoria donde
	// se guardarn los vrtices (memoria de video, agp, memoria de sistema...)
	// Finalmente le pasamos el buffer de vrtices al cual se le asignar una
	// zona en memoria.
	if(FAILED(g_pD3DDev->CreateVertexBuffer(18 * sizeof(MIVERTICE),
											0,
											D3DFVF_MIVERTICE,
											D3DPOOL_DEFAULT,
											&g_pVB,
											NULL)))
	{
		return E_FAIL;
	}

	VOID* pVertices;

	// Siempre que queramos aadir vrtices a nuestro buffer, debemos bloquear el
	// buffer.
	if(FAILED(g_pVB->Lock(0, sizeof(vertices), &pVertices, 0)))
		return E_FAIL;

	// copiamos los vrtices en los que hemos especificado el cubo
	// a la zona de memoria a la que apunta nuestro buffer de vrtices.
	memcpy(pVertices, vertices, sizeof(vertices));

	// Nunca olvidar desbloquear el buffer cuando se halla terminado de rellenarlo!
	g_pVB->Unlock();

	return S_OK;
}

//-----------------------------------------------------------------------------
// Funcin: ActualizaMatrices()
// Propsito: Especifica las matrices de mundo, de vista y de proyeccin
//-----------------------------------------------------------------------------
void ActualizaMatrices(void)
{
	// especificamos matrices de rotacin en el eje X, Y, y Z para nuestra
	// matriz de mundo. Esto har que el cubo rote alrededor
	// de los ejes del mundo 3D.
    D3DXMATRIX matWorld, matWorldX, matWorldY, matWorldZ;
    
	//creamos las matrices de transformacin
	D3DXMatrixRotationX(&matWorldX, 2*D3DX_PI*(timeGetTime()*0.0001f));
	D3DXMatrixRotationY(&matWorldY, 2*D3DX_PI*(timeGetTime()*0.0001f));	
	D3DXMatrixRotationZ(&matWorldZ, 2*D3DX_PI*(timeGetTime()*0.0001f));	

	//Las combinamos multiplicandolas unas con otras
	D3DXMatrixMultiply(&matWorld, &matWorldX, &matWorldY);
	D3DXMatrixMultiply(&matWorld, &matWorld, &matWorldZ);

	//Aplicamos la transformacion a la matriz de mundo
    g_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);

	//Aqu crearemos la cmara
	//La cmara tiene tres paramateros: "Posicion", "Donde Mira" y "Direccion Arriba"
	//Hemos puesto lo siguiente:
	//Posicion:			(0, 0, -30)
	//Donde Mira:		(0, 0, 0)
	//Direccion Arriba:	Eje-Y.
    D3DXMATRIX matView;
    D3DXMatrixLookAtLH(&matView, &D3DXVECTOR3(0.0f, 0.0f,-30.0f),	//Posicion
                                 &D3DXVECTOR3(0.0f, 0.0f, 0.0f),	//Donde Mira
                                 &D3DXVECTOR3(0.0f, 1.0f, 0.0f));	//Direccion Arriba
    g_pD3DDev->SetTransform(D3DTS_VIEW, &matView);

	// Ahora creamos una matriz de proyeccin la cual proyectar nuestros objetos
	// 3D en un plano (2D). Una matriz de proyeccin se puede definir a travs de
	// un FOV (field of view - campo de visin) el cual es un ngulo que 
	// especificaremos en radianes. En este caso el FOV es de 45, por un aspect ratio
	// o aspecto que indica la relacin entre el ancho y el alto de la ventana (en las
	// pantallas normales tenemos una relacion 4:3 asique el ratio ser 1.33333), un plano
	// cercano que especifica en qu punto cercano al observador se cortar la escena, es
	// decir, no se dibujar y finalmente un plano lejano que indica en qu punto se dejar
	// de dibujar la escena ms all del observador.
    D3DXMATRIX matProj;
    D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, WND_WIDTH / (float)WND_HEIGHT, 1.0f, 500.0f);
    g_pD3DDev->SetTransform(D3DTS_PROJECTION, &matProj);
}

//----------------------------------------------------------------------------
// Funcin: FinalizarD3D
// Propsito: Elimina todos los objetos D3D inicializados anteriormente...por ese orden!
//----------------------------------------------------------------------------
void FinalizarD3D(void)
{
	if(g_pVB != NULL)
		g_pVB->Release();

	if(g_pD3DDev != NULL)
		g_pD3DDev->Release();

	if(g_pD3D != NULL)
		g_pD3D->Release();
}

//----------------------------------------------------------------------------
// Funcin: Render
// Propsito: Aqu se pondrn nuestras funciones de render
//----------------------------------------------------------------------------
void Render(void)
{
	if(NULL == g_pD3DDev)
		return;

	// Limpia la pantalla de un color (negro en este caso)
	g_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(0,0,0), 1.0f, 0);
    
	// Indica a D3D que en esta seccin pondremos nuestras 
	// funciones de render
	g_pD3DDev->BeginScene();


    //---------------------------------------//

	ActualizaMatrices();
    // Especificamos cual es la fuente de la cual sacaremos la geometra
	g_pD3DDev->SetStreamSource(0, g_pVB, 0, sizeof(MIVERTICE));
	// Seleccionamos un tipo de vrtice...en este caso el que hemos 
	// creado nosotros
	g_pD3DDev->SetFVF(D3DFVF_MIVERTICE);
	// Dibujamos los vrtices..en este caso los dibujamos de forma que se
	// interpreten como listas de tringulos, le decimos que comience en 
	// el vrtice n 0 y que tan slo vamos a dibujar una primitiva del
	// tipo tringulo.
	g_pD3DDev->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);	//Arriba
	g_pD3DDev->DrawPrimitive(D3DPT_TRIANGLESTRIP, 4, 8);	//Lados
	g_pD3DDev->DrawPrimitive(D3DPT_TRIANGLESTRIP, 14,2);	//Abajo
	

	//---------------------------------------//

    
	// A partir de aqu ya no podemos poner ms funciones
	// de render
	g_pD3DDev->EndScene();
    
	// Finalmente se pasa el contenido 
	// del backbuffer al dispositivo.
	g_pD3DDev->Present(NULL, NULL, g_hWnd, NULL);
}

//-------------------------------------------------------------------------------
// Funcin: MsgProc
// Propsito: Manejar los mensajes de la aplicacin
//-------------------------------------------------------------------------------
LRESULT WINAPI MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_DESTROY:
		
			PostQuitMessage(0);
			return 0;

		case WM_PAINT:

			ValidateRect(hWnd, NULL);
			return 0;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

//----------------------------------------------------------------------------
// Funcin: WinMain
// Propsito; Es la funcin principal de toda aplicacin de Windows
// Se encarga de crear la ventana, mostrarla y es la receptora de los
// mensajes que le son enviados a la aplicacin
//----------------------------------------------------------------------------
INT WINAPI WinMain(HINSTANCE hInstance,			// Instancia a nuestra aplicacin
				   HINSTANCE hPrevInstance,		// Existen instancias previas de nuestra aplicacin?
				   LPSTR lpCmdLine,				// Lnea de comandos
				   int nShowCmd)				// Cmo se despliega nuestra ventana?
{
	
	if(!SUCCEEDED(IniVentana(hInstance)))
		return 0;

	if(SUCCEEDED(InicializarD3D(g_hWnd)))
	{
		if(SUCCEEDED(IniGeometria()))
		{
			// Mostrar nuestra ventana
			ShowWindow(g_hWnd, nShowCmd);
			UpdateWindow(g_hWnd);

			// Si est en modo ventana mostramos el cursor
			if(g_bWindowed)
				ShowCursor(TRUE);
			else	// de lo contrario lo ocultamos
				ShowCursor(FALSE);

			// Bucle de mensajes tpico de Windows
			MSG	msg;	// handle a un mensaje
			ZeroMemory(&msg, sizeof(msg));

			while(msg.message != WM_QUIT)
			{
				// si se pulsa ESC salimos
				if(GET_KEYDOWN(VK_ESCAPE))
					PostQuitMessage(0);

				if(PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE))
				{
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
				else
				{
					Render();
				}
			}
		}
	}

    // Salir de la aplicacin
	FinalizarD3D();
	UnregisterClass(APP_NAME,hInstance);

	return (0);
}