//-------------------------------------------------------------------------------
// CJuego.cpp
//-------------------------------------------------------------------------------

#include "CJuego.h"

//-------------------------------------------------------------------------------
// Funcin: Constructor
//-------------------------------------------------------------------------------

CJuego::CJuego(HWND Wnd,void *DebInHandle,void *DebOutHandle)
{
	hWnd=Wnd;
	debugInputHandle=DebInHandle;
	debugOutputHandle=DebOutHandle;
	Util= new CUtil(hWnd,debugInputHandle,debugOutputHandle);
	Cubo1= new CCubo();
	m_pD3D = NULL;
	m_pD3DDev = NULL;
	
}

//-------------------------------------------------------------------------------
// Funcin: Destructor
//-------------------------------------------------------------------------------

CJuego::~CJuego()
{
	BorrarInter(m_pD3DDev);
	BorrarInter(m_pD3D);
	BorrarObj(Cubo1);
	Util->EscribirLog("-> Direct3D finalizado correctamente.\n"); 
    BorrarObj(Util);
}

//----------------------------------------------------------------------------
// Funcin: SelModoVideo
// Propsito: Selecciona el mejor modo de video posible
//----------------------------------------------------------------------------

D3DFORMAT CJuego::SelModoVideo(UINT nAncho, UINT nAlto, UINT nBits)
{
	UINT x;
	D3DDISPLAYMODE d3ddm;

	for(x = 0; x < m_pD3D->GetAdapterModeCount(0, D3DFMT_X8R8G8B8); x++)
	{
		m_pD3D->EnumAdapterModes(0, D3DFMT_X8R8G8B8, x, &d3ddm);
		if(d3ddm.Width == nAncho)
		{
			if(d3ddm.Height == nAlto)
			{
				if((d3ddm.Format == D3DFMT_R5G6B5) || (d3ddm.Format == D3DFMT_X1R5G5B5) || (d3ddm.Format == D3DFMT_X4R4G4B4))
				{
					if(nBits == 16)
					{
						return d3ddm.Format;
					}
				}
				else if((d3ddm.Format == D3DFMT_R8G8B8) || (d3ddm.Format == D3DFMT_X8R8G8B8))
				{
					if(nBits == 32)
					{
						return d3ddm.Format;
					}
				}
			}
		}
	}

	return D3DFMT_UNKNOWN;
}

//----------------------------------------------------------------------------
// Funcin: InicializarD3D
// Propsito: Inicia el objeto D3D y el dispositivo D3D
//----------------------------------------------------------------------------
HRESULT CJuego::InicializarD3D(BOOL bWindowed)
{
	// En primer lugar creamos el objeto D3D
	if((m_pD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
	{
		Util->EscribirLog("\t*- Error al crear el objeto D3D.");
		return E_FAIL;
	}

	Util->EscribirLog("\t-> Objeto D3D inicializado (m_pD3D).");

	D3DDISPLAYMODE d3ddm;
	// Especificamos como dispositivo de visualizacin el dispositivo primario
	
	d3ddm.Format=SelModoVideo(WND_WIDTH,WND_HEIGHT,32);
	if(d3ddm.Format != D3DFMT_UNKNOWN)
	{
		//Ancho x Alto x 32bit seleccionado
		d3ddm.Width = WND_WIDTH;
		d3ddm.Height = WND_HEIGHT;

		Util->EscribirLog("\t-> Modo %dx%dx32bit seleccionado. Formato = %d.",WND_WIDTH,WND_HEIGHT,d3ddm.Format);
	}
	else
	{
		d3ddm.Format=SelModoVideo(WND_WIDTH,WND_HEIGHT,16);
		if(d3ddm.Format != D3DFMT_UNKNOWN)
		{
            //Ancho x Alto x 16bit seleccionado
			d3ddm.Width = WND_WIDTH;
			d3ddm.Height = WND_HEIGHT;

			Util->EscribirLog("\t-> Modo %dx%dx16bit seleccionado. Formato = %d.",WND_WIDTH,WND_HEIGHT,d3ddm.Format);
		}
        else
		{
			Util->EscribirLog("\t*- Imposible selecionar el modo de video para %dx%d",WND_WIDTH,WND_HEIGHT);
            return E_FAIL;
        }
	}

	// Ahora rellenamos la estructura usada para crear el dispositivo
	// Indicamos que queremos un backbuffer.
	// Ajustamos la anchura y la altura del backbuffer al tamao de nuestra
	// ventana.
	// Especificamos una aplicacin a pantalla completa.
	// Especificamos que D3D escoja el sistema ms ptimo para realizar
	// el intercambio del backbuffer a frontbuffer.
	// Especificamos un formato para el backbuffer igual que el de nuestro
	// escritorio.
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp, sizeof(d3dpp));
	d3dpp.BackBufferCount = 1;
	d3dpp.BackBufferWidth = WND_WIDTH;
	d3dpp.BackBufferHeight = WND_HEIGHT;
	d3dpp.hDeviceWindow = hWnd;
	d3dpp.Windowed = bWindowed;
	d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	
	if (!bWindowed)
	{		
		d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
		d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
	}
	else
	{
		d3dpp.PresentationInterval = 0;
		d3dpp.FullScreen_RefreshRateInHz = 0;		
	}
			
	//Seleccionamos el mejor buffer de profundidad, 32, 24 o 16 bit
    if(m_pD3D->CheckDeviceFormat(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,d3ddm.Format,D3DUSAGE_DEPTHSTENCIL,D3DRTYPE_SURFACE,D3DFMT_D32)==D3D_OK)
	{
        d3dpp.AutoDepthStencilFormat = D3DFMT_D32;
        d3dpp.EnableAutoDepthStencil = TRUE;

		Util->EscribirLog("\t-> Seleccionado 32bit depth buffer.");
    }
    else if(m_pD3D->CheckDeviceFormat(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,d3ddm.Format,D3DUSAGE_DEPTHSTENCIL,D3DRTYPE_SURFACE,D3DFMT_D24X8)==D3D_OK)
    {
		d3dpp.AutoDepthStencilFormat = D3DFMT_D24X8;
        d3dpp.EnableAutoDepthStencil = TRUE;

		Util->EscribirLog("\t-> Seleccionado 24bit depth buffer.");
	}
    else if(m_pD3D->CheckDeviceFormat(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,d3ddm.Format,D3DUSAGE_DEPTHSTENCIL,D3DRTYPE_SURFACE,D3DFMT_D16)==D3D_OK)
    {
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
        d3dpp.EnableAutoDepthStencil = TRUE;

		Util->EscribirLog("\t-> Seleccionado 16bit depth buffer.");
	}
    else
	{
        d3dpp.EnableAutoDepthStencil = FALSE;
		Util->EscribirLog("\t*- Imposible seleccionar un depth buffer.");
	}

	// Por ltimo creamos el dispositivo D3D (la aceleradora 3D propiamente dicha)
	// Escogemos el dispositivo por defecto (el primario)
	// Especificamos que queremos un dispositivo HAL, es decir, que tenga aceleracin
	// a traves de D3D.
	// Especificamos la ventana que se asociar al dispositivo.
	// Especificamos que queremos que D3D procese los vrtices por software...esto
	// se debera comprobar ya que si disponemos de un dispositivo TnL tal como
	// GeForce, preferiremos que se realice por hardware.
	if(FAILED(m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
									D3DCREATE_SOFTWARE_VERTEXPROCESSING,
									&d3dpp, &m_pD3DDev)))
	{
		Util->EscribirLog("\t*- No se pudo crear el dispositivo D3D.\n");
		return E_FAIL;
	}
	
	Util->EscribirLog("\t-> Dispositivo D3D creado (m_pD3DDev).\n");
	
	//Activa el back face culling. Lo hacemos porque no queremos que se pinte las
	//caras de los poligonos que no se ven
    m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

	//Desactivamos la luz porque la estemos especificando en el color de vrtice
    m_pD3DDev->SetRenderState(D3DRS_LIGHTING, FALSE);

	return S_OK;
}


//----------------------------------------------------------------------------
// Funcin: InicializarJuego
// Propsito: Inicializa los datos del juego
//----------------------------------------------------------------------------
HRESULT CJuego::InicializarJuego()
{
	Util->EscribirLog("-> Inicializando datos del juego...\n");
	

	if (FAILED(Cubo1->Inicializar(m_pD3DDev)))
	{
		Util->EscribirLog("\t*- Error al iniciar Cubo1!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Cubo1 inicializado correctamente.");

	if (FAILED(Cubo1->Textura("gfx\\textura.bmp")))
	{
		Util->EscribirLog("\t*- Error al cargar la textura para Cubo1!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Textura cargada para Cubo1.");
	
	Cubo1->Posicion(0,0,0);
	Cubo1->Dimensiones(15,15,15);

	//Ponemos el filtrado bilinear de texturas
	m_pD3DDev->SetSamplerState(0,D3DSAMP_MINFILTER,	D3DTEXF_LINEAR);
	m_pD3DDev->SetSamplerState(0,D3DSAMP_MAGFILTER,	D3DTEXF_LINEAR);
	
	return S_OK;
}

//-----------------------------------------------------------------------------
// Funcin: ActualizaMatrices()
// Propsito: Especifica las matrices de mundo, de vista y de proyeccin
//-----------------------------------------------------------------------------
void CJuego::ActualizaMatrices(void)
{
	// especificamos matrices de rotacin en el eje X, Y, y Z para nuestra
	// matriz de mundo. Esto har que el cubo rote alrededor
	// de los ejes del mundo 3D.
    D3DXMATRIX matWorld, matWorldX, matWorldY, matWorldZ;
    
	//creamos las matrices de transformacin
	D3DXMatrixRotationX(&matWorldX, timeGetTime()/2000.0f);
	D3DXMatrixRotationY(&matWorldY, timeGetTime()/2000.0f);	
	D3DXMatrixRotationZ(&matWorldZ, timeGetTime()/2000.0f);	

	//Las combinamos multiplicandolas unas con otras
	D3DXMatrixMultiply(&matWorld, &matWorldX, &matWorldY);
	D3DXMatrixMultiply(&matWorld, &matWorld, &matWorldZ);

	//Aplicamos la transformacion a la matriz de mundo
    m_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);

	//Aqu crearemos la cmara
	//La cmara tiene tres paramateros: "Posicion", "Donde Mira" and "Direccion Arriba"
	//Hemos puesto lo siguiente:
	//Posicion:			(0, 0, -30)
	//Donde Mira:		(0, 0, 0)
	//Direccion Arriba:	Y-Axis.
    D3DXMATRIX matView;
    D3DXMatrixLookAtLH(&matView, &D3DXVECTOR3(0.0f, 0.0f,-30.0f),	//Posicion
                                 &D3DXVECTOR3(0.0f, 0.0f, 0.0f),	//Donde Mira
                                 &D3DXVECTOR3(0.0f, 1.0f, 0.0f));	//Direccion Arriba
    m_pD3DDev->SetTransform(D3DTS_VIEW, &matView);

	// Ahora creamos una matriz de proyeccin la cual proyectar nuestros objetos
	// 3D en un plano (2D). Una matriz de proyeccin se puede definir a travs de
	// un FOV (field of view - campo de visin) el cual es un ngulo que 
	// especificaremos en radianes. En este caso el FOV es de 45, por un aspect ratio
	// o aspecto que indica la relacin entre el ancho y el alto de la ventana (en las
	// pantallas normales tenemos una relacion 4:3 asique el ratio ser 1.33333), un plano
	// cercano que especifica en qu punto cercano al observador se cortar la escena, es
	// decir, no se dibujar y finalmente un plano lejano que indica en qu punto se dejar
	// de dibujar la escena ms all del observador.
    D3DXMATRIX matProj;
    D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, WND_WIDTH / (float)WND_HEIGHT, 1.0f, 500.0f);
    m_pD3DDev->SetTransform(D3DTS_PROJECTION, &matProj);
}

//----------------------------------------------------------------------------
// Funcin: BuclePpal
// Propsito: Este es el bucle principal del juego
//----------------------------------------------------------------------------
void CJuego::BuclePpal(void)
{
	if(NULL == m_pD3DDev)
		return;

	// Limpia la pantalla de un color (negro en este caso)
	m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,0), 1.0f, 0);
    
	// Indica a D3D que en esta seccin pondremos nuestras 
	// funciones de render
	m_pD3DDev->BeginScene();


    //---------------------------------------//

	ActualizaMatrices();
	Cubo1->Renderizar();

	//---------------------------------------//

    
	// A partir de aqu ya no podemos poner ms funciones
	// de render
	m_pD3DDev->EndScene();
    
	// Finalmente se pasa el contenido 
	// del backbuffer al dispositivo.
	m_pD3DDev->Present(NULL, NULL, hWnd, NULL);
}

//-------------------------------------------------------------------------------
// Funcin: EscribirConsola
// Propsito: Escribe en la consola
//-------------------------------------------------------------------------------

void CJuego::EscribirConsola(char* text, ...)
{
	if (debugOutputHandle)
	{
		static char Buffer[1024];
		va_list args;
		va_start(args,text);
		vsprintf(Buffer,text,args);
		va_end(args);

		DWORD num;
		if (!Buffer)
			return;

		WriteConsole(debugOutputHandle,Buffer,strlen(Buffer),&num,NULL);
	}
}

