//-------------------------------------------------------------------------------
// Tutorial 4 de Direct3D 9
// PLANET ISO
// http://www.planetiso.cjb.net
//-------------------------------------------------------------------------------

#include "principal.h"
#include "CJuego.h"
#include "CUtil.h"

//-------------------------------------------------------------------------------
// Declaracin de funciones
//-------------------------------------------------------------------------------
LRESULT WINAPI	MsgProc(HWND, UINT, WPARAM, LPARAM);
INT WINAPI		WinMain(HINSTANCE, HINSTANCE, LPSTR, int);

HRESULT			IniVentana(HINSTANCE hInstance);

void			ActivarConsola(void);
void			DesactivarConsola(void);
void			EscribirConsola(char* text, ...);

//-----------------------------------------------------------------------------
// Declaracin de variabes globales
//-----------------------------------------------------------------------------
HWND					g_hWnd;				// Nuestra ventana
CJuego					*Juego=NULL;
CUtil					*Util=NULL;
void					*debugInputHandle;
void					*debugOutputHandle;
BOOL					g_bWindowed = TRUE;	// A pantalla completa o en ventana?
BOOL					g_bDebug=TRUE;
BOOL					g_bConsola=FALSE;

//-------------------------------------------------------------------------------
// Funcin: ActivarConsola
// Propsito: Muestra una consola
//-------------------------------------------------------------------------------

void ActivarConsola(void)
{
	AllocConsole();
	SetConsoleTitle(APP_NAME" Debug");
	debugInputHandle = GetStdHandle(STD_INPUT_HANDLE);
	debugOutputHandle = GetStdHandle(STD_OUTPUT_HANDLE);
	g_bConsola=TRUE;
}

//-------------------------------------------------------------------------------
// Funcin: DesactivarConsola
// Propsito: Quita la consola
//-------------------------------------------------------------------------------

void DesactivarConsola(void)
{
	AllocConsole();
	SetConsoleTitle(APP_NAME" Debug");
	debugInputHandle = GetStdHandle(STD_INPUT_HANDLE);
	debugOutputHandle = GetStdHandle(STD_OUTPUT_HANDLE);
	g_bConsola=FALSE;
}

//-------------------------------------------------------------------------------
// Funcin: EscribirConsola
// Propsito: Escribe en la consola
//-------------------------------------------------------------------------------

void EscribirConsola(char* text, ...)
{
	if (debugOutputHandle)
	{
		static char Buffer[1024];
		va_list args;
		va_start(args,text);
		vsprintf(Buffer,text,args);
		va_end(args);

		DWORD num;
		if (!Buffer)
			return;

		WriteConsole(debugOutputHandle,Buffer,strlen(Buffer),&num,NULL);
	}
}

//----------------------------------------------------------------------------
// Funcin: IniVentana
// Propsito: Inicializa nuestra ventana
//----------------------------------------------------------------------------
HRESULT IniVentana(HINSTANCE hInstance)
{
	
	WNDCLASSEX	wc;		// clase de ventana
	
	// Rellenamos la estructura de clase de ventana
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_CLASSDC;
	wc.lpfnWndProc = MsgProc;
	wc.cbClsExtra = 0L;
	wc.cbWndExtra = 0L;
	wc.hInstance = GetModuleHandle(NULL);
	wc.hIcon = NULL;
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = APP_NAME;
	wc.hIconSm = NULL;


	// Registramos la clase de vetana
	if(!RegisterClassEx(&wc))
	{
		MessageBox(NULL, "No se pudo registrar la clase de ventana", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}
	
	// Creamos la ventana
	g_hWnd = CreateWindowEx(
						NULL,					// no se suele usar
						APP_NAME,				// nombre de la clase
						APP_NAME,				// ttulo de la ventana
						WS_OVERLAPPEDWINDOW,	// tipo de ventana
						CW_USEDEFAULT,			// posicin X de la ventana
						CW_USEDEFAULT,			// posicin Y de a ventana
						WND_WIDTH,				// ancho de la ventana
						WND_HEIGHT,				// alto de la ventana
						GetDesktopWindow(),		// ventana padre
						NULL,					// men
						wc.hInstance,			// instancia de la aplicacin
						NULL);					// no se suele usar
	
	if (!g_hWnd)
    {
        MessageBox(NULL,"No se cre el objeto g_hWnd","Error",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
    }

	return S_OK;
}

//-------------------------------------------------------------------------------
// Funcin: MsgProc
// Propsito: Manejar los mensajes de la aplicacin
//-------------------------------------------------------------------------------
LRESULT WINAPI MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_DESTROY:
		
			PostQuitMessage(0);
			return 0;

		case WM_PAINT:

			ValidateRect(hWnd, NULL);
			return 0;
		
		case WM_SIZE:

			EscribirConsola(" Cambiando el aspecto de la ventana. . .\n");
			break;

		case WM_MOVE:

			EscribirConsola(" Moviendo la ventana. . .\n");
			break;

			
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

//----------------------------------------------------------------------------
// Funcin: WinMain
// Propsito; Es la funcin principal de toda aplicacin de Windows
// Se encarga de crear la ventana, mostrarla y es la receptora de los
// mensajes que le son enviados a la aplicacin
//----------------------------------------------------------------------------
INT WINAPI WinMain(HINSTANCE hInstance,			// Instancia a nuestra aplicacin
				   HINSTANCE hPrevInstance,		// Existen instancias previas de nuestra aplicacin?
				   LPSTR lpCmdLine,				// Lnea de comandos
				   int nShowCmd)				// Cmo se despliega nuestra ventana?
{
	char hora[128];
	char fecha[128];
	struct tm *today;
	time_t ltime;

	if(!SUCCEEDED(IniVentana(hInstance)))
		return 0;

	if(g_bDebug)
		ActivarConsola();

	EscribirConsola("!!!IMPORTANTE NO CERRAR ESTA VENTANA!!!\n\n");

	Util= new CUtil(g_hWnd,debugInputHandle,debugOutputHandle);
	Juego= new CJuego(g_hWnd,debugInputHandle,debugOutputHandle);

	_strtime(hora);
	_strdate(fecha);
	today=localtime(&ltime);
	
	Util->ActivarLog();
	Util->EscribirLog("************ %s Logbook ************  %s - %s  ****\n",
		APP_NAME,hora,fecha);
	
	Util->EscribirLog("-> Objeto g_hWnd registrado. Ventana Creada.\n");
	
	if(g_bConsola)
		Util->EscribirLog("-> Consola activada.\n");

	if(g_bWindowed)
		Util->EscribirLog("-> Procediendo a incializar D3D en modo ventana...\n");	
	else
		Util->EscribirLog("-> Procediendo a incializar D3D en pantalla completa...\n");
	
	if (SUCCEEDED(Juego->InicializarD3D(g_bWindowed)))
	{
		Util->EscribirLog("-> Direct3D Inicializado.\n");
		
		// Si est en modo ventana mostramos el cursor
		if(g_bWindowed)
		{
			ShowCursor(TRUE);
			Util->EscribirLog("-> Cursor activado.\n");
		}		
		else	// de lo contrario lo ocultamos
		{
			ShowCursor(FALSE);
			Util->EscribirLog("-> Cursor desactivado.\n");
		}

		// Mostrar nuestra ventana
		ShowWindow(g_hWnd, nShowCmd);
		UpdateWindow(g_hWnd);

		Util->EscribirLog("-> Ventana mostrada.\n");

		if (SUCCEEDED(Juego->InicializarJuego()))
		{

			// Bucle de mensajes tpico de Windows
			MSG	msg;	// handle a un mensaje
			ZeroMemory(&msg, sizeof(msg));

			Util->EscribirLog("   ***************************************************\n");
			Util->EscribirLog("\t-> Ejecutando la aplicacin . . .");

			while(msg.message != WM_QUIT)
			{
				// si se pulsa ESC salimos
				if(GET_KEYDOWN(VK_ESCAPE))
					PostQuitMessage(0);

				if(PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE))
				{
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
				else
				{
					Juego->BuclePpal();
				}
			}

			Util->EscribirLog("\t-> Ejecucin finalizada.\n");
			Util->EscribirLog("   ***************************************************\n");
		}
	}

    // Salir de la aplicacin

	if(g_bConsola)
	{
		DesactivarConsola();
		Util->EscribirLog("-> Consola desactivada.\n");
	}

	BorrarObj(Juego);

	UnregisterClass(APP_NAME,hInstance);

	Util->EscribirLog("-> Objeto g_hWnd eliminado. Ventana Finalizada.\n");
	
	Util->EscribirLog("-> Aplicacin Terminada.\n");

	_strtime(hora);
	_strdate(fecha);

	Util->EscribirLog("********** %s Fin Logbook **********  %s - %s  ****\n",
		APP_NAME,hora,fecha);

	BorrarObj(Util);

	return 0;
}