//-------------------------------------------------------------------------------
// CCubo.cpp
//-------------------------------------------------------------------------------

#include "CCubo.h"


//------------------------------------------------------------------------------
// Funcin: Constructor
//------------------------------------------------------------------------------
CCubo::CCubo(void)
{
	m_pTextura=NULL;
	BufferVert = NULL;
}

HRESULT CCubo::Inicializar(LPDIRECT3DDEVICE9 pD3DDevice)
{
	D3DDisp = pD3DDevice;
	
	//Ponemos posicion y tamao estandar
	Ancho = 10.0;
	Alto = 10.0;
	Largo = 10.0;
	m_rX = 0.0;
	m_rY = 0.0;
	m_rZ = 0.0;

	//Initcializamos el Vertex Buffer
    if(SUCCEEDED(CrearBufferVert()))
	{
		ActualizarVertices();
	}
	else
		return E_FAIL;

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: Destructor
//------------------------------------------------------------------------------
CCubo::~CCubo()
{
	BorrarInter(m_pTextura);
	BorrarInter(BufferVert);
}

//------------------------------------------------------------------------------
// Funcin: Textura
// Propsito: Carga la textura de un archivo
//------------------------------------------------------------------------------
HRESULT CCubo::Textura(const char *szArchivo)
{
	if(FAILED(D3DXCreateTextureFromFile(D3DDisp, szArchivo, &m_pTextura)))
	{
		return E_FAIL;
	}

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: Renderizar
// Propsito: Renderiza el cubo
//------------------------------------------------------------------------------
BOOL CCubo::Renderizar()
{
	CargarEstados();

	D3DDisp->SetStreamSource(0, BufferVert, 0, sizeof(VERTCUBO));
    D3DDisp->SetFVF(D3DFVF_VERTCUBO);
	
	if(m_pTextura)
	{
		//Hay textura. No queremeos que la textura se sombree segun los colores
		//de los vertices y para ello ponemos D3DTOP_SELECTARG1
        D3DDisp->SetTexture(0, m_pTextura);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
    }
	else
	{
		//No hay textura. Desactivamos el renderizado con textura.
		D3DDisp->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
	}

    D3DDisp->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);		//Top
	D3DDisp->DrawPrimitive(D3DPT_TRIANGLESTRIP, 4, 8);		//Sides
	D3DDisp->DrawPrimitive(D3DPT_TRIANGLESTRIP, 14, 2);		//Bottom

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: CargarEstados
// Propsito: Carga los estados necesarios para pintar el cubo en condiciones
//------------------------------------------------------------------------------
void CCubo::CargarEstados(void)
{
	// Especificamos los estados de render y texture-stages que necesitamos
	D3DDisp->SetRenderState(D3DRS_ALPHABLENDENABLE,			TRUE);
	D3DDisp->SetRenderState(D3DRS_SRCBLEND,					D3DBLEND_SRCALPHA);
	D3DDisp->SetRenderState(D3DRS_DESTBLEND,				D3DBLEND_INVSRCALPHA);
	D3DDisp->SetRenderState(D3DRS_ALPHATESTENABLE,			FALSE);
	D3DDisp->SetRenderState(D3DRS_FILLMODE,					D3DFILL_SOLID);
	D3DDisp->SetRenderState(D3DRS_CULLMODE,					D3DCULL_CCW);
	D3DDisp->SetRenderState(D3DRS_LIGHTING,					FALSE);
	D3DDisp->SetRenderState(D3DRS_ZENABLE,					TRUE);
	D3DDisp->SetRenderState(D3DRS_STENCILENABLE,			TRUE);
	D3DDisp->SetRenderState(D3DRS_CLIPPING,					TRUE);
	D3DDisp->SetRenderState(D3DRS_CLIPPLANEENABLE,			FALSE);
	D3DDisp->SetRenderState(D3DRS_VERTEXBLEND,				FALSE);
	D3DDisp->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE,	FALSE);
	D3DDisp->SetRenderState(D3DRS_FOGENABLE,				FALSE);

	D3DDisp->SetTextureStageState(0,D3DTSS_COLORARG1,				D3DTA_TEXTURE);
	D3DDisp->SetTextureStageState(0,D3DTSS_COLORARG2,				D3DTA_DIFFUSE);
	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAARG1,				D3DTA_TEXTURE);
	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAOP,					D3DTOP_MODULATE);
	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAARG2,				D3DTA_DIFFUSE);
	D3DDisp->SetTextureStageState(0,D3DTSS_TEXCOORDINDEX,			D3DTSS_TCI_PASSTHRU);
	D3DDisp->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,	D3DTTFF_DISABLE);
	D3DDisp->SetTextureStageState(1,D3DTSS_COLOROP,					D3DTOP_DISABLE);
	D3DDisp->SetTextureStageState(1,D3DTSS_ALPHAOP,					D3DTOP_DISABLE);

	D3DDisp->SetSamplerState(0,D3DSAMP_MINFILTER,					D3DTEXF_LINEAR);
	D3DDisp->SetSamplerState(0,D3DSAMP_MAGFILTER,					D3DTEXF_LINEAR);
	D3DDisp->SetSamplerState(0,D3DSAMP_MIPFILTER,					D3DTEXF_LINEAR);

}

//------------------------------------------------------------------------------
// Funcin: CrearBufferVert
// Propsito: Crea el buffer de vrtices del cubo
//------------------------------------------------------------------------------
HRESULT CCubo::CrearBufferVert()
{
    // Creamos el buffer de vrtices.
	// Primero especificamos el tamao...18 vrtices * el tamao de la estructura
	// que hemos definido.
	// Seguidamente especificamos el modo de uso.
	// El siguiente argumento es el tipo de vrtice.
	// Seguidamente especificamos que queremos que D3D elija la memoria donde
	// se guardarn los vrtices (memoria de video, agp, memoria de sistema...)
	// Finalmente le pasamos el buffer de vrtices al cual se le asignar una
	// zona en memoria.
	if(FAILED(D3DDisp->CreateVertexBuffer(18 * sizeof(VERTCUBO),
											0,
											D3DFVF_VERTCUBO,
											D3DPOOL_DEFAULT,
											&BufferVert,
											NULL)))
	{
        return E_FAIL;
    }

    return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: ActualizarVertices
// Propsito: Actualiza los cambios en los vrtices
//------------------------------------------------------------------------------
BOOL CCubo::ActualizarVertices()
{
	VOID* pVertices;
    
    // asignamos vrtices que forman un cubo con un color para cada vrtice
	// hay que asegurarse que los vertices se definen en el sentido de
	// las agujas del reloj ya que las caras contrarias las eliminamos con el culling
    VERTCUBO cvVertices[] =
    {	
		//Cara Superior
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(0, 0, 255), 0.0f, 1.0f,},		//Vertex 0 - Blue 
        {m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), D3DCOLOR_XRGB(255, 0, 0), 0.0f, 0.0f,},		//Vertex 1 - Red 
        {m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(255, 0, 0), 1.0f, 1.0f,},		//Vertex 2 - Red 
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), D3DCOLOR_XRGB(0, 255, 0), 1.0f, 0.0f,},		//Vertex 3 - Green 

		//Cara 1
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(255, 0, 0), 0.0f, 1.0f,},		//Vertex 4 - Red 
        {m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(0, 0, 255), 0.0f, 0.0f,},		//Vertex 5 - Blue 
        {m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(0, 255, 0), 1.0f, 1.0f,},		//Vertex 6 - Green 
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(255, 0, 0), 1.0f, 0.0f,},		//Vertex 7 - Red 

		//Cara 2
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), D3DCOLOR_XRGB(0, 0, 255), 0.0f, 1.0f,},		//Vertex 8 - Blue 
        {m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), D3DCOLOR_XRGB(0, 255, 0), 0.0f, 0.0f,},		//Vertex 9 - Green
		
		//Cara 3
        {m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), D3DCOLOR_XRGB(0, 255, 0), 1.0f, 1.0f,},		//Vertex 10 - Green 
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), D3DCOLOR_XRGB(255, 0, 0), 1.0f, 0.0f,},		//Vertex 11 - Red 

		//Cara 4
        {m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(255, 0, 0), 0.0f, 1.0f,},		//Vertex 12 - Red 
        {m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(0, 0, 255), 0.0f, 0.0f,},		//Vertex 13 - Blue

		//Cara Inferior
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(0, 255, 0), 0.0f, 1.0f,},		//Vertex 14 - Green 
        {m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), D3DCOLOR_XRGB(0, 0, 255), 0.0f, 0.0f,},		//Vertex 15 - Blue 
        {m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), D3DCOLOR_XRGB(255, 0, 0), 1.0f, 1.0f,},		//Vertex 16 - Red 
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), D3DCOLOR_XRGB(0, 255, 0), 1.0f, 0.0f,},		//Vertex 17 - Green
	 };

	// Siempre que queramos aadir vrtices a nuestro buffer, debemos bloquear el
	// buffer.
    if(FAILED(BufferVert->Lock(0, sizeof(cvVertices), &pVertices, 0)))
    {
        return FALSE;
    }

    // copiamos los vrtices en los que hemos especificado el cubo
	// a la zona de memoria a la que apunta nuestro buffer de vrtices.
    memcpy(pVertices, cvVertices, sizeof(cvVertices));

    // Nunca olvidar desbloquear el buffer cuando se halla terminado de rellenarlo!
    BufferVert->Unlock();

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: Dimensiones
// Propsito: Pone las dimensiones del cubo
//------------------------------------------------------------------------------
BOOL CCubo::Dimensiones(float rAncho, float rAlto, float rLargo)
{
	Ancho = rAncho;
	Alto = rAlto;
	Largo = rLargo;

	ActualizarVertices();

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: ActualizarVertices
// Propsito: Pone el origen del cubo
//------------------------------------------------------------------------------
BOOL CCubo::Posicion(float x, float y, float z)
{
	m_rX = x;
	m_rY = y;
	m_rZ = z;

	ActualizarVertices();

	return TRUE;
}