//-------------------------------------------------------------------------------
// CCubo.cpp
//-------------------------------------------------------------------------------

#include "CCubo.h"


//------------------------------------------------------------------------------
// Funcin: Constructor
//------------------------------------------------------------------------------
CCubo::CCubo(void)
{
	m_pTextura=NULL;
	BufferVert = NULL;
}

HRESULT CCubo::Inicializar(LPDIRECT3DDEVICE9 pD3DDevice)
{
	D3DDisp = pD3DDevice;
	
	//Ponemos posicion y tamao estandar
	Ancho = 10.0;
	Alto = 10.0;
	Largo = 10.0;
	m_rX = 0.0;
	m_rY = 0.0;
	m_rZ = 0.0;

	//ponemos un material estandar (R, G, B, A)
	D3DCOLORVALUE rgbaDifusa  = {1.0, 1.0, 1.0, 0.0,};
	D3DCOLORVALUE rgbaAmbiente  = {1.0, 1.0, 1.0, 0.0,};
	D3DCOLORVALUE rgbaEspecular = {0.0, 0.0, 0.0, 0.0,};
	D3DCOLORVALUE rgbaEmisiva = {0.0, 0.0, 0.0, 0.0,};
	
	Material(rgbaDifusa, rgbaAmbiente, rgbaEspecular, rgbaEmisiva, 0);

	//Initcializamos el Vertex Buffer
    if(SUCCEEDED(CrearBufferVert()))
	{
		ActualizarVertices();
	}
	else
		return E_FAIL;

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: Destructor
//------------------------------------------------------------------------------
CCubo::~CCubo()
{
	BorrarInter(m_pTextura);
	BorrarInter(BufferVert);
}

//------------------------------------------------------------------------------
// Funcin: Textura
// Propsito: Carga la textura de un archivo
//------------------------------------------------------------------------------
HRESULT CCubo::Textura(const char *szArchivo)
{
	if(FAILED(D3DXCreateTextureFromFile(D3DDisp, szArchivo, &m_pTextura)))
	{
		return E_FAIL;
	}

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: Material
// Propsito: Inicializa el material para el cubo
//------------------------------------------------------------------------------
BOOL CCubo::Material(D3DCOLORVALUE rgbaDifuso, D3DCOLORVALUE rgbaAmbiente, D3DCOLORVALUE rgbaEspecular, D3DCOLORVALUE rgbaEmisivo, float rPotencia)
{
	m_matMaterial.Diffuse = rgbaDifuso; 

	m_matMaterial.Ambient = rgbaAmbiente; 

	m_matMaterial.Specular = rgbaEspecular; 

	m_matMaterial.Power = rPotencia;

	m_matMaterial.Emissive = rgbaEmisivo;

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: Renderizar
// Propsito: Renderiza el cubo
//------------------------------------------------------------------------------
BOOL CCubo::Renderizar()
{
	CargarEstados();

	D3DDisp->SetStreamSource(0, BufferVert, 0, sizeof(VERTCUBO));
    D3DDisp->SetFVF(D3DFVF_VERTCUBO);
	
	if(m_pTextura)
	{
		//Hay textura. Queremeos que la textura se sombree segun las luces
		//y para ello ponemos D3DTOP_MODULATE.
		D3DDisp->SetTexture(0, m_pTextura);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);

    }
	else
	{
		//No hay textura. Desactivamos el renderizado con textura.
		D3DDisp->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
	}

	if(FAILED(D3DDisp->SetMaterial(&m_matMaterial)))
	{
		//Fallo al iniciar el material
		return FALSE;
	}
	
	D3DDisp->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 12);

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: CargarEstados
// Propsito: Carga los estados necesarios para pintar el cubo en condiciones
//------------------------------------------------------------------------------
void CCubo::CargarEstados(void)
{
	// Especificamos los estados de render y texture-stages que necesitamos
	D3DDisp->SetRenderState(D3DRS_ALPHABLENDENABLE,			TRUE);
	D3DDisp->SetRenderState(D3DRS_SRCBLEND,					D3DBLEND_ONE);
	D3DDisp->SetRenderState(D3DRS_DESTBLEND,				D3DBLEND_ZERO);
	D3DDisp->SetRenderState(D3DRS_ALPHATESTENABLE,			FALSE);
	D3DDisp->SetRenderState(D3DRS_FILLMODE,					D3DFILL_SOLID);
	D3DDisp->SetRenderState(D3DRS_CULLMODE,					D3DCULL_CCW);
	D3DDisp->SetRenderState(D3DRS_LIGHTING,					TRUE);
	D3DDisp->SetRenderState(D3DRS_ZENABLE,					TRUE);
	D3DDisp->SetRenderState(D3DRS_STENCILENABLE,			TRUE);
	D3DDisp->SetRenderState(D3DRS_CLIPPING,					TRUE);
	D3DDisp->SetRenderState(D3DRS_CLIPPLANEENABLE,			FALSE);
	D3DDisp->SetRenderState(D3DRS_VERTEXBLEND,				FALSE);
	D3DDisp->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE,	FALSE);
	D3DDisp->SetRenderState(D3DRS_FOGENABLE,				FALSE);

	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAARG1,				D3DTA_TEXTURE);
	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAARG2,				D3DTA_DIFFUSE);
	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAOP,					D3DTOP_MODULATE);
	D3DDisp->SetTextureStageState(0,D3DTSS_TEXCOORDINDEX,			D3DTSS_TCI_PASSTHRU);
	D3DDisp->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,	D3DTTFF_DISABLE);
	D3DDisp->SetTextureStageState(1,D3DTSS_COLOROP,					D3DTOP_DISABLE);
	D3DDisp->SetTextureStageState(1,D3DTSS_ALPHAOP,					D3DTOP_DISABLE);

	D3DDisp->SetSamplerState(0,D3DSAMP_MINFILTER,					D3DTEXF_LINEAR);
	D3DDisp->SetSamplerState(0,D3DSAMP_MAGFILTER,					D3DTEXF_LINEAR);
	D3DDisp->SetSamplerState(0,D3DSAMP_MIPFILTER,					D3DTEXF_LINEAR);

}

//------------------------------------------------------------------------------
// Funcin: CrearBufferVert
// Propsito: Crea el buffer de vrtices del cubo
//------------------------------------------------------------------------------
HRESULT CCubo::CrearBufferVert()
{
    
	if(FAILED(D3DDisp->CreateVertexBuffer(36 * sizeof(VERTCUBO),
											0,
											D3DFVF_VERTCUBO,
											D3DPOOL_DEFAULT,
											&BufferVert,
											NULL)))
	{
        return E_FAIL;
    }

    return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: ActualizarVertices
// Propsito: Actualiza los cambios en los vrtices
//------------------------------------------------------------------------------
BOOL CCubo::ActualizarVertices()
{
	VOID* pVertices;
	D3DVECTOR vNormal;
    
    // asignamos vrtices que forman un cubo con un color para cada vrtice
	// hay que asegurarse que los vertices se definen en el sentido de
	// las agujas del reloj ya que las caras contrarias las eliminamos con el culling
    VERTCUBO cvVertices[] =
    {	
		//Cara Superior
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f,},		//Vertex 0
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 1
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 2
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 0.0f,},		//Vertex 3
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 4
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 5

		//Cara 1
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f,},		//Vertex 6
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 7
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 8
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 0.0f,},		//Vertex 9
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 10
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 11


		//Cara 2
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f,},		//Vertex 12
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 13
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 14
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 0.0f,},		//Vertex 15
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 16
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 17
		

		//Cara 3
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f,},		//Vertex 18
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 19
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 20
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 0.0f,},		//Vertex 21
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 22
		{m_rX + (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 23


		//Cara 4
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f,},		//Vertex 24
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 25
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 26
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 0.0f,},		//Vertex 27
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 28
		{m_rX - (Ancho / 2), m_rY + (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 29


		//Cara Inferior
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f,},		//Vertex 30
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 31
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 32
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 0.0f,},		//Vertex 33
		{m_rX - (Ancho / 2), m_rY - (Alto / 2), m_rZ - (Largo / 2), 0.0f, 0.0f, 0.0f, 1.0f, 1.0f,},		//Vertex 34
		{m_rX + (Ancho / 2), m_rY - (Alto / 2), m_rZ + (Largo / 2), 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,},		//Vertex 35
	};

	//Poner las normales
	int i;

	for(i = 0; i < 36; i += 3)
	{
		vNormal = DaNormalTriangulo(&D3DXVECTOR3(cvVertices[i].x, cvVertices[i].y, cvVertices[i].z), &D3DXVECTOR3(cvVertices[i + 1].x, cvVertices[i + 1].y, cvVertices[i + 1].z), &D3DXVECTOR3(cvVertices[i + 2].x, cvVertices[i + 2].y, cvVertices[i + 2].z));

		cvVertices[i].nx = vNormal.x;
		cvVertices[i].ny = vNormal.y;
		cvVertices[i].nz = vNormal.z;

		cvVertices[i + 1].nx = vNormal.x;
		cvVertices[i + 1].ny = vNormal.y;
		cvVertices[i + 1].nz = vNormal.z;

		cvVertices[i + 2].nx = vNormal.x;
		cvVertices[i + 2].ny = vNormal.y;
		cvVertices[i + 2].nz = vNormal.z;

	}


    if(FAILED(BufferVert->Lock(0, sizeof(cvVertices), &pVertices, 0)))
    {
        return FALSE;
    }

    memcpy(pVertices, cvVertices, sizeof(cvVertices));

    BufferVert->Unlock();

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: DaNormalTriangulo
// Propsito: Devuelve la normal de un tringulo
//------------------------------------------------------------------------------
D3DVECTOR CCubo::DaNormalTriangulo(D3DXVECTOR3* vVertice1, D3DXVECTOR3* vVertice2, D3DXVECTOR3* vVertice3)
{
	D3DXVECTOR3 vNormal;
	D3DXVECTOR3 v1;
	D3DXVECTOR3 v2;

	D3DXVec3Subtract(&v1, vVertice2, vVertice1);
	D3DXVec3Subtract(&v2, vVertice3, vVertice1);

	D3DXVec3Cross(&vNormal, &v1, &v2);

	D3DXVec3Normalize(&vNormal, &vNormal);

	return vNormal;
}

//------------------------------------------------------------------------------
// Funcin: Dimensiones
// Propsito: Pone las dimensiones del cubo
//------------------------------------------------------------------------------
BOOL CCubo::Dimensiones(float rAncho, float rAlto, float rLargo)
{
	Ancho = rAncho;
	Alto = rAlto;
	Largo = rLargo;

	ActualizarVertices();

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: ActualizarVertices
// Propsito: Pone el origen del cubo
//------------------------------------------------------------------------------
BOOL CCubo::Posicion(float x, float y, float z)
{
	m_rX = x;
	m_rY = y;
	m_rZ = z;

	ActualizarVertices();

	return TRUE;
}