//-------------------------------------------------------------------------------
// CJuego.cpp
//-------------------------------------------------------------------------------

#include "CJuego.h"

//-------------------------------------------------------------------------------
// Funcin: Constructor
//-------------------------------------------------------------------------------

CJuego::CJuego(HWND Wnd,void *DebInHandle,void *DebOutHandle)
{
	hWnd=Wnd;
	debugInputHandle=DebInHandle;
	debugOutputHandle=DebOutHandle;
	Util= new CUtil(hWnd,debugInputHandle,debugOutputHandle);
	Crono= new CCrono();
	Camara= new CCamara();
	Input= new CInput();
	Fuente= new CFuente(hWnd,debugInputHandle,debugOutputHandle);
	Cubo1= new CCubo();
	Cubo2= new CCubo();
	Cubo3= new CCubo();
	Cubo4= new CCubo();
	Destello= new CPanel();
	Suelo= new CPanel();

	m_pD3D = NULL;
	m_pD3DDev = NULL;
	
}

//-------------------------------------------------------------------------------
// Funcin: Destructor
//-------------------------------------------------------------------------------

CJuego::~CJuego()
{
	BorrarInter(m_pD3DDev);
	BorrarInter(m_pD3D);
	Crono->Parar();
	BorrarObj(Crono);
	BorrarObj(Input);
	BorrarObj(Camara);
	BorrarObj(Fuente);
	BorrarObj(Cubo1);
	BorrarObj(Cubo2);
	BorrarObj(Cubo3);
	BorrarObj(Cubo4);
	BorrarObj(Destello);
	BorrarObj(Suelo);

	Util->EscribirLog("\n-> Direct3D finalizado correctamente.\n"); 
    BorrarObj(Util);
}

//----------------------------------------------------------------------------
// Funcin: SelModoVideo
// Propsito: Selecciona el mejor modo de video posible
//----------------------------------------------------------------------------

D3DFORMAT CJuego::SelModoVideo(UINT nAncho, UINT nAlto, UINT nBits)
{
	UINT x;
	D3DDISPLAYMODE d3ddm;

	for(x = 0; x < m_pD3D->GetAdapterModeCount(0, D3DFMT_X8R8G8B8); x++)
	{
		m_pD3D->EnumAdapterModes(0, D3DFMT_X8R8G8B8, x, &d3ddm);
		if(d3ddm.Width == nAncho)
		{
			if(d3ddm.Height == nAlto)
			{
				if((d3ddm.Format == D3DFMT_R5G6B5) || (d3ddm.Format == D3DFMT_X1R5G5B5) || (d3ddm.Format == D3DFMT_X4R4G4B4) || (d3ddm.Format == D3DFMT_A8R3G3B2))
				{
					if(nBits == 16)
					{
						return d3ddm.Format;
					}
				}
				else if((d3ddm.Format == D3DFMT_R8G8B8) || (d3ddm.Format == D3DFMT_X8R8G8B8))
				{
					if(nBits == 32)
					{
						return d3ddm.Format;
					}
				}
			}
		}
	}

	return D3DFMT_UNKNOWN;
}

//----------------------------------------------------------------------------
// Funcin: InicializarD3D
// Propsito: Inicia el objeto D3D y el dispositivo D3D
//----------------------------------------------------------------------------
HRESULT CJuego::InicializarD3D(BOOL bWindowed)
{

	int bits=0;

	// En primer lugar creamos el objeto D3D
	if((m_pD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
	{
		Util->EscribirLog("\t*- Error al crear el objeto D3D.\n");
		return E_FAIL;
	}

	Util->EscribirLog("\t-> Objeto D3D inicializado (m_pD3D).");

	D3DDISPLAYMODE d3ddm;
	// Especificamos como dispositivo de visualizacin el dispositivo primario
	
	if(FAILED(m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
		return E_FAIL;
	
	d3ddm.Format=SelModoVideo(WND_WIDTH,WND_HEIGHT,32);
	if(d3ddm.Format != D3DFMT_UNKNOWN)
	{
		//Ancho x Alto x 32bit seleccionado
		d3ddm.Width = WND_WIDTH;
		d3ddm.Height = WND_HEIGHT;

		Util->EscribirLog("\t-> Modo %dx%dx32bit seleccionado. Formato ARGB = %d.",WND_WIDTH,WND_HEIGHT,d3ddm.Format);
		
		bits=32;
	}
	else
	{
		d3ddm.Format=SelModoVideo(WND_WIDTH,WND_HEIGHT,16);
		if(d3ddm.Format != D3DFMT_UNKNOWN)
		{
			//Ancho x Alto x 16bit seleccionado
			d3ddm.Width = WND_WIDTH;
			d3ddm.Height = WND_HEIGHT;

			Util->EscribirLog("\t-> Modo %dx%dx16bit seleccionado. Formato = %d.",WND_WIDTH,WND_HEIGHT,d3ddm.Format);

			bits=16;
		}
		else
		{
			Util->EscribirLog("\t*- Imposible seleccionar el modo de video para %dx%d\n",WND_WIDTH,WND_HEIGHT);
			return E_FAIL;
		}
	}

	// Ahora rellenamos la estructura usada para crear el dispositivo
	// Indicamos que queremos un backbuffer.
	// Ajustamos la anchura y la altura del backbuffer al tamao de nuestra
	// ventana.
	// Especificamos una aplicacin a pantalla completa.
	// Especificamos que D3D escoja el sistema ms ptimo para realizar
	// el intercambio del backbuffer a frontbuffer.
	// Especificamos un formato para el backbuffer igual que el de nuestro
	// escritorio.
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp, sizeof(d3dpp));
	d3dpp.BackBufferCount = 1;
	d3dpp.BackBufferWidth = WND_WIDTH;
	d3dpp.BackBufferHeight = WND_HEIGHT;
	d3dpp.hDeviceWindow = hWnd;
	d3dpp.Windowed = bWindowed;
	d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	
	if (!bWindowed)
	{		
		d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
		d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
	}
	else
	{
		d3dpp.PresentationInterval = 0;
		d3dpp.FullScreen_RefreshRateInHz = 0;		
	}

	
	//Seleccionamos el mejor buffer de profundidad, 32, 24 o 16 bit
    if(m_pD3D->CheckDeviceFormat(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,d3ddm.Format,D3DUSAGE_DEPTHSTENCIL,D3DRTYPE_SURFACE,D3DFMT_D32)==D3D_OK && bits>16)
	{
        d3dpp.AutoDepthStencilFormat = D3DFMT_D32;
        d3dpp.EnableAutoDepthStencil = TRUE;

		Util->EscribirLog("\t-> Seleccionado 32bit depth buffer.");
    }
    else if(m_pD3D->CheckDeviceFormat(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,d3ddm.Format,D3DUSAGE_DEPTHSTENCIL,D3DRTYPE_SURFACE,D3DFMT_D24X8)==D3D_OK && bits>16)
    {
		d3dpp.AutoDepthStencilFormat = D3DFMT_D24X8;
        d3dpp.EnableAutoDepthStencil = TRUE;

		Util->EscribirLog("\t-> Seleccionado 24bit depth buffer.");
	}
    else if(m_pD3D->CheckDeviceFormat(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,d3ddm.Format,D3DUSAGE_DEPTHSTENCIL,D3DRTYPE_SURFACE,D3DFMT_D16)==D3D_OK)
    {
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
        d3dpp.EnableAutoDepthStencil = TRUE;

		Util->EscribirLog("\t-> Seleccionado 16bit depth buffer.");
	}
    else
	{
	    d3dpp.EnableAutoDepthStencil = FALSE;
		Util->EscribirLog("\t*- Imposible seleccionar un depth buffer.");
	}

	// Por ltimo creamos el dispositivo D3D (la aceleradora 3D propiamente dicha)
	// Escogemos el dispositivo por defecto (el primario)
	// Especificamos que queremos un dispositivo HAL, es decir, que tenga aceleracin
	// a traves de D3D.
	// Especificamos la ventana que se asociar al dispositivo.
	// Especificamos que queremos que D3D procese los vrtices por software...esto
	// se debera comprobar ya que si disponemos de un dispositivo TnL tal como
	// GeForce, preferiremos que se realice por hardware.
	if(FAILED(m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
									D3DCREATE_HARDWARE_VERTEXPROCESSING,
									&d3dpp, &m_pD3DDev)))
	{
		Util->EscribirLog("\t*- No se pudo crear el dispositivo D3D T&L.");

		if(FAILED(m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
									D3DCREATE_SOFTWARE_VERTEXPROCESSING,
									&d3dpp, &m_pD3DDev)))
		{
			Util->EscribirLog("\t*- No se pudo crear el dispositivo D3D normal.");

			if(FAILED(m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, hWnd,
									D3DCREATE_SOFTWARE_VERTEXPROCESSING,
									&d3dpp, &m_pD3DDev)))
			{
				Util->EscribirLog("\t*- No se pudo crear el dispositivo D3D software.");
				return E_FAIL;
			}
			else
				Util->EscribirLog("\t-> Dispositivo D3D (software) creado (m_pD3DDev).\n");
		}
		else
			Util->EscribirLog("\t-> Dispositivo D3D (normal) creado (m_pD3DDev).\n");
	}
	else
		Util->EscribirLog("\t-> Dispositivo D3D (T&L) creado (m_pD3DDev).\n");
	
	return S_OK;
}

//----------------------------------------------------------------------------
// Funcin: InicializarLuces
// Propsito: Inicializa las luces del juego
//----------------------------------------------------------------------------
BOOL CJuego::InicializarLuces(void)
{
	//Inicializar la estructura a cero
	ZeroMemory(&Luz, sizeof(D3DLIGHT9));

	//Ponemos eun punto de luz blanca en (0, 0, 0).
	Luz.Type = D3DLIGHT_POINT;

	Luz.Diffuse.r = 1.0f;
	Luz.Diffuse.g = 1.0f;
	Luz.Diffuse.b = 1.0f;

	Luz.Ambient.r = 0.0f;
	Luz.Ambient.g = 0.0f;
	Luz.Ambient.b = 0.0f;
	
	Luz.Specular.r = 0.0f;
	Luz.Specular.g	= 1.0f;
	Luz.Specular.b	= 0.0f;

	Luz.Position.x	= 0.0f;
	Luz.Position.y	= 0.0f;
	Luz.Position.z	= 0.0f;

	Luz.Attenuation0 = 1.0f; 
	Luz.Attenuation1 = 0.0f; 
	Luz.Attenuation2 = 0.0f; 

	Luz.Range = 250.0f;

	//Asignamos el punto de luz al dispositivo en la posicion 0
	if(FAILED(m_pD3DDev->SetLight(0, &Luz)))
	{
		return FALSE;
	}
	

	// "encendemos" la luz de  la posicin 0 (la que asignamos antes)
	if(FAILED(m_pD3DDev->LightEnable(0, TRUE)))
	{
		return FALSE;
	}
	

	//Ponemos el nivel de luz ambiental
	if(FAILED(m_pD3DDev->SetRenderState(D3DRS_AMBIENT, D3DCOLOR_XRGB(64,64,64))))	
	{
		return FALSE;
	}
	

	return TRUE;
}

//----------------------------------------------------------------------------
// Funcin: ActualizarLuces
// Propsito: Inicializa las luces del juego
//----------------------------------------------------------------------------
BOOL CJuego::ActualizarLuces(void)
{
	

	Luz.Position.x=0;
	Luz.Position.y=25*sinf(timeGetTime()/700.0f);
	Luz.Position.z=0;

	if(FAILED(m_pD3DDev->SetLight(0, &Luz)))
	{
		return FALSE;
	}
	

	return TRUE;
}


//----------------------------------------------------------------------------
// Funcin: InicializarJuego
// Propsito: Inicializa los datos del juego
//----------------------------------------------------------------------------
HRESULT CJuego::InicializarJuego()
{
	Util->EscribirLog("-> Inicializando datos del juego...\n");

	//Inicializamos las luces
	InicializarLuces();

	//Inicializamos la entrada de datos por teclado y raton
	//el raton en modo NO EXCLUSIVO
	//y el teclado en modo EXCLUSIVO
	Input->Inicializar(hWnd,FALSE,FALSE);

	//Activa el back face culling. Lo hacemos porque no queremos que se pinte las
	//caras de los poligonos que no se ven
    m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
	
	//Activamos el buffer Z
	m_pD3DDev->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);


	if (FAILED(Fuente->CargarFuente(m_pD3DDev,"gfx\\fuente.tga")))
		return E_FAIL;

	//Inicilizamos los 4 cubos
	/////////////////////////////////////////////////////////////
	
	if (FAILED(Cubo1->Inicializar(m_pD3DDev)))
	{
		Util->EscribirLog("\t*- Error al iniciar Cubo1!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Cubo1 inicializado correctamente.");

	if (FAILED(Cubo1->Textura("gfx\\textura.tga")))
	{
		Util->EscribirLog("\t*- Error al cargar la textura para Cubo1!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Textura cargada para Cubo1.");
	
	Cubo1->Posicion(0,0,0);
	Cubo1->Dimensiones(15,15,15);

	/////////////////////////////////////////////////////////////

	if (FAILED(Cubo2->Inicializar(m_pD3DDev)))
	{
		Util->EscribirLog("\t*- Error al iniciar Cubo2!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Cubo2 inicializado correctamente.");

	if (FAILED(Cubo2->Textura("gfx\\textura.tga")))
	{
		Util->EscribirLog("\t*- Error al cargar la textura para Cubo2!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Textura cargada para Cubo2.");
	
	Cubo2->Posicion(0,0,0);
	Cubo2->Dimensiones(15,15,15);

	/////////////////////////////////////////////////////////////

	if (FAILED(Cubo3->Inicializar(m_pD3DDev)))
	{
		Util->EscribirLog("\t*- Error al iniciar Cubo3!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Cubo3 inicializado correctamente.");

	if (FAILED(Cubo3->Textura("gfx\\textura2.bmp")))
	{
		Util->EscribirLog("\t*- Error al cargar la textura para Cubo3!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Textura cargada para Cubo3.");
	
	Cubo3->Posicion(0,0,0);
	Cubo3->Dimensiones(15,15,15);

	/////////////////////////////////////////////////////////////

	if (FAILED(Cubo4->Inicializar(m_pD3DDev)))
	{
		Util->EscribirLog("\t*- Error al iniciar Cubo4!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Cubo4 inicializado correctamente.");

	if (FAILED(Cubo4->Textura("gfx\\textura2.bmp")))
	{
		Util->EscribirLog("\t*- Error al cargar la textura para Cubo4!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Textura cargada para Cubo4.");
	
	Cubo4->Posicion(0,0,0);
	Cubo4->Dimensiones(15,15,15);

	/////////////////////////////////////////////////////////////
	
	//Inicilizamos el destello que sera un panel de 1 segmento
	//con el canal alpha activado
	/////////////////////////////////////////////////////////////

	if (FAILED(Destello->Inicializar(m_pD3DDev,1)))
	{
		Util->EscribirLog("\t*- Error al iniciar Destello!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Destello inicializado correctamente.");

	if (FAILED(Destello->Textura("gfx\\destello.tga")))
	{
		Util->EscribirLog("\t*- Error al cargar la textura para Destello!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Textura cargada para Destello.");

	Destello->CanalAlpha(TRUE);
	Destello->Posicion(0,0,0);
	Destello->Dimensiones(40,40);

	/////////////////////////////////////////////////////////////

	//Inicilizamos el suelo que sera un panel de 10 segmentos
	//con la iluminacion activada
	/////////////////////////////////////////////////////////////

	if (FAILED(Suelo->Inicializar(m_pD3DDev,10)))
	{
		Util->EscribirLog("\t*- Error al iniciar Suelo!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Suelo inicializado correctamente.");

	if (FAILED(Suelo->Textura("gfx\\suelo.jpg")))
	{
		Util->EscribirLog("\t*- Error al cargar la textura para Suelo!!");
		return E_FAIL;
	}
	else
		Util->EscribirLog("\t-> Textura cargada para Suelo.");
	
	Suelo->Iluminado(TRUE);
	Suelo->Posicion(0,0,0);
	Suelo->Dimensiones(300,300);

	//Empezamos el cronometro del juego
	Crono->Empezar();

	return S_OK;
}

//-----------------------------------------------------------------------------
// Funcin: ActualizaCamara()
// Propsito: Especifica las matrices de mundo, de vista y de proyeccin
//-----------------------------------------------------------------------------
void CJuego::ActualizaCamara(void)
{

	if(Input->BotonPulsado(BOTON_IZQUIERDO))
	{
		Camara->Subir(20,Crono->DaTiempoDelta());
	}
	if(Input->BotonPulsado(BOTON_DERECHO))
	{
		Camara->Subir(-20,Crono->DaTiempoDelta());
	}

	// Rotacin de la cmara /////////////////
	D3DXVECTOR3 angles = Camara->DaOrientacion();


	// Cogemos la posicin relativa del ratn
	POINTS Pos;
	Pos.x = Input->XRelativaRaton();
	Pos.y = Input->YRelativaRaton();

	// Calculamos la nueva orientacin de la cmara
	// El pitch
	angles.x += D3DXToRadian((float)Pos.y);
	if(angles.x > D3DXToRadian(90.0f))
		angles.x = D3DXToRadian(90.0f);
	else if(angles.x < D3DXToRadian(-90.0f))
		angles.x = D3DXToRadian(-90.0f);

	// El yaw
	angles.y += D3DXToRadian((float)Pos.x);
	if(angles.y >= D3DXToRadian(360.0f))
		angles.y = 0.0f;
	else if(angles.y <= D3DXToRadian(-360.0f))
		angles.y = 0.0f;

	// Actualizamos los ngulos de la cmara
	Camara->PonerOrientacion(angles.x, angles.y, 0.0f);
	///////////////////////////////////////////

	Camara->ActualizarVista(m_pD3DDev);
	Camara->ActualizarProyeccion(m_pD3DDev);
}

//----------------------------------------------------------------------------
// Funcin: BuclePpal
// Propsito: Este es el bucle principal del juego
//----------------------------------------------------------------------------
void CJuego::BuclePpal(void)
{
	if(NULL == m_pD3DDev)
		return;

	Crono->Frame();

	Input->Actualizar();

	// Limpia la pantalla de un color (negro en este caso)
	m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,0), 1.0f, 0);
    
	// Indica a D3D que en esta seccin pondremos nuestras 
	// funciones de render
	m_pD3DDev->BeginScene();


    //---------------------------------------//
	
	//Actualizamos los movimientos de camara
	ActualizaCamara();

	//Matrices que usaremos
    D3DXMATRIX matWorld, matWorldA, matWorldB, matWorldT;

	//Matrices de rotacion y traslacion para los distintos ejes
	D3DXMatrixRotationY(&matWorldA, timeGetTime()/-500.0f);	
	D3DXMatrixRotationY(&matWorldB, timeGetTime()/2000.0f);
	D3DXMatrixTranslation(&matWorldT,-70,0,0);	

	//Las multimplicamos siguiendo un orden
	//(el orden de los factores en matrices SI altera el producto)
	D3DXMatrixMultiply(&matWorldA, &matWorldA, &matWorldT);
	D3DXMatrixMultiply(&matWorld, &matWorldA, &matWorldB);

	//Aplicamos la transformacion final a la matriz mundo
    m_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);

	//Renderizamos el cubo1
	Cubo1->Renderizar();

	///////////////////////

	D3DXMatrixRotationY(&matWorldA, timeGetTime()/-500.0f);		
	D3DXMatrixRotationY(&matWorldB, timeGetTime()/2000.0f);
	D3DXMatrixTranslation(&matWorldT,70,0,0);	

	D3DXMatrixMultiply(&matWorldA, &matWorldA, &matWorldT);
	D3DXMatrixMultiply(&matWorld, &matWorldA, &matWorldB);

    m_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);

	//Renderizamos el cubo2
	Cubo2->Renderizar();
	
	///////////////////////

	D3DXMatrixRotationY(&matWorldA, timeGetTime()/1000.0f);	
	D3DXMatrixRotationY(&matWorldB, timeGetTime()/2000.0f);
	D3DXMatrixTranslation(&matWorldT,0,0,70);	

	D3DXMatrixMultiply(&matWorldA, &matWorldA, &matWorldT);
	D3DXMatrixMultiply(&matWorld, &matWorldA, &matWorldB);

    m_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);

	//Renderizamos el cubo3
	Cubo3->Renderizar();

	///////////////////////

	D3DXMatrixRotationY(&matWorldA, timeGetTime()/1000.0f);	
	D3DXMatrixRotationY(&matWorldB, timeGetTime()/2000.0f);
	D3DXMatrixTranslation(&matWorldT,0,0,-70);	

	D3DXMatrixMultiply(&matWorldA, &matWorldA, &matWorldT);
	D3DXMatrixMultiply(&matWorld, &matWorldA, &matWorldB);

    m_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);

	//Renderizamos el cubo4
	Cubo4->Renderizar();

	///////////////////////
	D3DXMatrixRotationX(&matWorldA, D3DXToRadian(0));
	D3DXMatrixTranslation(&matWorldT,-(Suelo->Ancho/2),-50,-(Suelo->Alto/2));
	D3DXMatrixMultiply(&matWorld, &matWorldA, &matWorldT);
	
	m_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);
	
	//Renderizamos el suelo
	Suelo->Renderizar();

	///////////////////////

	D3DXMatrixRotationX(&matWorldA, D3DXToRadian(90));
	D3DXMatrixRotationZ(&matWorldB, timeGetTime()/500.0f);
	D3DXMatrixTranslation(&matWorldT,-(Destello->Ancho/2),(Destello->Alto/2),0);
	
	//Tambien se pueden multiplicar utilizando el operador * que esta sobrecargado
	//en las librerias de D3DX
	matWorldA=matWorldA*matWorldT;
	
	D3DXMatrixTranslation(&matWorldT,0,25*sinf(timeGetTime()/700.0f),0);

	matWorld=matWorldA*matWorldB*matWorldT;
	
	m_pD3DDev->SetTransform(D3DTS_WORLD, &matWorld);

	//Renderizamos el destello
	Destello->Renderizar();

	///////////////////////

	//Actulizampos la posicin de las luces
	ActualizarLuces();

	///////////////////////

	//Cambiamos los aspectos de la fuente
	//y pintamos los FPS obtenidos de nuestro cronometro
	Fuente->PonerSombra(TRUE);
	Fuente->PonerColor(D3DCOLOR_XRGB(255,255,255));
	Fuente->Dimensiones(12,12);
	Fuente->PintaTextoXY(0,0,"%.2f",Crono->DaFps());

	//Cambiamos los aspectos de la fuente
	//y pintamos el texto "Direct3D" en el fondo de la pantalla
	Fuente->PonerSombra(FALSE);
	Fuente->PonerColor(D3DCOLOR_ARGB(128,255,0,0));
	Fuente->Dimensiones(40,40);
	Fuente->PintaLineaXY(WND_WIDTH/2,WND_HEIGHT-50,TEXTO_CENTRO,"Direct3D");


	//---------------------------------------//

    
	// A partir de aqu ya no podemos poner ms funciones
	// de render
	m_pD3DDev->EndScene();
    
	// Finalmente se pasa el contenido 
	// del backbuffer al dispositivo.
	m_pD3DDev->Present(NULL, NULL, hWnd, NULL);
}

//-------------------------------------------------------------------------------
// Funcin: EscribirConsola
// Propsito: Escribe en la consola
//-------------------------------------------------------------------------------

void CJuego::EscribirConsola(char* text, ...)
{
	if (debugOutputHandle)
	{
		static char Buffer[1024];
		va_list args;
		va_start(args,text);
		vsprintf(Buffer,text,args);
		va_end(args);

		DWORD num;
		if (!Buffer)
			return;

		WriteConsole(debugOutputHandle,Buffer,strlen(Buffer),&num,NULL);
	}
}

