//-------------------------------------------------------------------------------
// CPanel.cpp
//-------------------------------------------------------------------------------

#include "CPanel.h"


//------------------------------------------------------------------------------
// Funcin: Constructor
//------------------------------------------------------------------------------
CPanel::CPanel(void)
{
	m_pTextura=NULL;
	BufferVert = NULL;
}

HRESULT CPanel::Inicializar(LPDIRECT3DDEVICE9 pD3DDevice,int iSegmentos)
{
	D3DDisp = pD3DDevice;
	
	//Poner posicion y tamao por defecto
	Ancho = 10.0;
	Alto = 10.0;
	m_rX = 0.0;
	m_rY = 0.0;
	m_rZ = 0.0;

	m_iSegmentos=iSegmentos;

	m_bIluminado=FALSE;
	m_bCanalAlpha=FALSE;

	//Poner los valores del material por defecto (R, G, B, A)
	D3DCOLORVALUE rgbaDifusa  = {1.0, 1.0, 1.0, 0.0,};
	D3DCOLORVALUE rgbaAmbiente  = {1.0, 1.0, 1.0, 0.0,};
	D3DCOLORVALUE rgbaEspecular = {0.0, 0.0, 0.0, 0.0,};
	D3DCOLORVALUE rgbaEmisiva = {0.0, 0.0, 0.0, 0.0,};
	
	Material(rgbaDifusa, rgbaAmbiente, rgbaEspecular, rgbaEmisiva, 0);

	//Inicializar el bfer de vrtices
    if(SUCCEEDED(CrearBufferVert()))
	{
		ActualizarVertices();
	}
	else
		return E_FAIL;

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: Destructor
//------------------------------------------------------------------------------
CPanel::~CPanel()
{
	BorrarInter(m_pTextura);
	BorrarInter(BufferVert);
}

//------------------------------------------------------------------------------
// Funcin: Textura
// Propsito: Carga la textura de un archivo
//------------------------------------------------------------------------------
HRESULT CPanel::Textura(const char *szArchivo)
{
	if(FAILED(D3DXCreateTextureFromFileEx(D3DDisp,
										  szArchivo,
										  D3DX_DEFAULT,
										  D3DX_DEFAULT,
										  10,
										  D3DUSAGE_RENDERTARGET,
										  D3DFMT_UNKNOWN,
										  D3DPOOL_DEFAULT,
										  D3DX_DEFAULT,
										  D3DX_DEFAULT,
										  0,
										  NULL,
										  NULL,
										  &m_pTextura)))
	{
		return E_FAIL;
	}

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: Material
// Propsito: Inicializa el material para el cubo
//------------------------------------------------------------------------------
BOOL CPanel::Material(D3DCOLORVALUE rgbaDifuso, D3DCOLORVALUE rgbaAmbiente, D3DCOLORVALUE rgbaEspecular, D3DCOLORVALUE rgbaEmisivo, float rPotencia)
{
	m_matMaterial.Diffuse = rgbaDifuso; 

	m_matMaterial.Ambient = rgbaAmbiente; 

	m_matMaterial.Specular = rgbaEspecular; 

	m_matMaterial.Power = rPotencia;

	m_matMaterial.Emissive = rgbaEmisivo;

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: Renderizar
// Propsito: Renderiza el cubo
//------------------------------------------------------------------------------
BOOL CPanel::Renderizar()
{
	CargarEstados();

	D3DDisp->SetStreamSource(0, BufferVert, 0, sizeof(VERTPANEL));
    D3DDisp->SetFVF(D3DFVF_VERTPANEL);
	
	if(m_pTextura)
	{
		//Hay textura. Queremeos que la textura se sombree segun las luces
		//y para ello ponemos D3DTOP_MODULATE.
		D3DDisp->SetTexture(0, m_pTextura);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);

    }
	else
	{
		//No hay textura. Desactivamos el renderizado con textura.
		D3DDisp->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		D3DDisp->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	}

	if(FAILED(D3DDisp->SetMaterial(&m_matMaterial)))
	{
		//Fallo al iniciar el material
		return FALSE;
	}
	
	D3DDisp->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2*m_iSegmentos*m_iSegmentos);

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: DaNormalTriangulo
// Propsito: Devuelve la normal de un tringulo
//------------------------------------------------------------------------------
D3DVECTOR CPanel::DaNormalTriangulo(D3DXVECTOR3* vVertice1, D3DXVECTOR3* vVertice2, D3DXVECTOR3* vVertice3)
{
	D3DXVECTOR3 vNormal;
	D3DXVECTOR3 v1;
	D3DXVECTOR3 v2;

	D3DXVec3Subtract(&v1, vVertice2, vVertice1);
	D3DXVec3Subtract(&v2, vVertice3, vVertice1);

	D3DXVec3Cross(&vNormal, &v1, &v2);

	D3DXVec3Normalize(&vNormal, &vNormal);

	return vNormal;
}

//------------------------------------------------------------------------------
// Funcin: CargarEstados
// Propsito: Carga los estados necesarios para pintar el cubo en condiciones
//------------------------------------------------------------------------------
void CPanel::CargarEstados(void)
{

	// Especificamos los estados de render y texture-stages que necesitamos
	D3DDisp->SetRenderState(D3DRS_ALPHABLENDENABLE,			m_bCanalAlpha);
	D3DDisp->SetRenderState(D3DRS_SRCBLEND,					D3DBLEND_SRCALPHA);
	D3DDisp->SetRenderState(D3DRS_DESTBLEND,				D3DBLEND_INVSRCALPHA);
	D3DDisp->SetRenderState(D3DRS_ALPHATESTENABLE,			FALSE);
	D3DDisp->SetRenderState(D3DRS_FILLMODE,					D3DFILL_SOLID);
	D3DDisp->SetRenderState(D3DRS_CULLMODE,					D3DCULL_NONE);
	D3DDisp->SetRenderState(D3DRS_LIGHTING,					m_bIluminado);
	D3DDisp->SetRenderState(D3DRS_ZENABLE,					TRUE);
	D3DDisp->SetRenderState(D3DRS_STENCILENABLE,			TRUE);
	D3DDisp->SetRenderState(D3DRS_CLIPPING,					TRUE);
	D3DDisp->SetRenderState(D3DRS_CLIPPLANEENABLE,			FALSE);
	D3DDisp->SetRenderState(D3DRS_VERTEXBLEND,				FALSE);
	D3DDisp->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE,	FALSE);
	D3DDisp->SetRenderState(D3DRS_FOGENABLE,				FALSE);

	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAARG1,				D3DTA_TEXTURE );
	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAARG2,				D3DTA_DIFFUSE );
	D3DDisp->SetTextureStageState(0,D3DTSS_ALPHAOP,					D3DTOP_MODULATE);
	D3DDisp->SetTextureStageState(0,D3DTSS_TEXCOORDINDEX,			D3DTSS_TCI_PASSTHRU);
	D3DDisp->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,	D3DTTFF_DISABLE);
	D3DDisp->SetTextureStageState(1,D3DTSS_COLOROP,					D3DTOP_DISABLE);
	D3DDisp->SetTextureStageState(1,D3DTSS_ALPHAOP,					D3DTOP_DISABLE);

	D3DDisp->SetSamplerState(0,D3DSAMP_MINFILTER,					D3DTEXF_LINEAR);
	D3DDisp->SetSamplerState(0,D3DSAMP_MAGFILTER,					D3DTEXF_LINEAR);
	D3DDisp->SetSamplerState(0,D3DSAMP_MIPFILTER,					D3DTEXF_LINEAR);

}

//------------------------------------------------------------------------------
// Funcin: Iluminado
// Propsito: Activa o desactiva la iluminacin del panel
//------------------------------------------------------------------------------
void CPanel::Iluminado(BOOL bIluminar)
{
	m_bIluminado=bIluminar;
	ActualizarVertices();
}

//------------------------------------------------------------------------------
// Funcin: CanalAlpha
// Propsito: Activa o desactiva el canal alpha
//------------------------------------------------------------------------------
void CPanel::CanalAlpha(BOOL bCanalAlpha)
{
	m_bCanalAlpha=bCanalAlpha;
}

//------------------------------------------------------------------------------
// Funcin: CrearBufferVert
// Propsito: Crea el buffer de vrtices del cubo
//------------------------------------------------------------------------------
HRESULT CPanel::CrearBufferVert()
{
    //Creamos el bfer de vrtices
	if(FAILED(D3DDisp->CreateVertexBuffer(m_iSegmentos * m_iSegmentos * 6 * sizeof(VERTPANEL),
											D3DUSAGE_WRITEONLY,
											D3DFVF_VERTPANEL,
											D3DPOOL_MANAGED,
											&BufferVert,
											NULL)))
											
	{
        return E_FAIL;
    }

    return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: ActualizarVertices
// Propsito: Actualiza los cambios en los vrtices
//------------------------------------------------------------------------------
BOOL CPanel::ActualizarVertices()
{
    VERTPANEL* cvVertices;

    if(FAILED(BufferVert->Lock(0, 0, (void**)&cvVertices, 0)))
    {
        return FALSE;
    }

	float dX = 1.0f/(m_iSegmentos);
    float dZ = 1.0f/(m_iSegmentos);
    long k = 0;
    for (long z=0; z < (m_iSegmentos); z++)
    {
        for (long x=0; x < (m_iSegmentos); x++)
        {
            cvVertices[k].p  = D3DXVECTOR3(Ancho * x*dX, 0.0f, Alto * z*dZ );
            cvVertices[k].n  = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
            k++;
            cvVertices[k].p  = D3DXVECTOR3(Ancho * x*dX, 0.0f, Alto * (z+1)*dZ );
            cvVertices[k].n  = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
            k++;
            cvVertices[k].p  = D3DXVECTOR3(Ancho * (x+1)*dX, 0.0f, Alto * (z+1)*dZ );
            cvVertices[k].n  = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
            k++;
            cvVertices[k].p  = D3DXVECTOR3(Ancho * x*dX, 0.0f, Alto * z*dZ );
            cvVertices[k].n  = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
            k++;
            cvVertices[k].p  = D3DXVECTOR3(Ancho * (x+1)*dX, 0.0f, Alto * (z+1)*dZ );
            cvVertices[k].n  = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
            k++;
            cvVertices[k].p  = D3DXVECTOR3(Ancho * (x+1)*dX, 0.0f, Alto * z*dZ );
            cvVertices[k].n  = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
            k++;
        }
    }

	for (long i=0;i<(m_iSegmentos*m_iSegmentos);i++)
	{
		cvVertices[0+(6*i)].u=0.0f;
		cvVertices[0+(6*i)].v=0.0f;
		cvVertices[1+(6*i)].u=0.0f;
		cvVertices[1+(6*i)].v=1.0f;
		cvVertices[2+(6*i)].u=1.0f;
		cvVertices[2+(6*i)].v=1.0f;
		cvVertices[3+(6*i)].u=0.0f;
		cvVertices[3+(6*i)].v=0.0f;
		cvVertices[4+(6*i)].u=1.0f;
		cvVertices[4+(6*i)].v=1.0f;
		cvVertices[5+(6*i)].u=1.0f;
		cvVertices[5+(6*i)].v=0.0f;
	}

    BufferVert->Unlock();

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: Dimensiones
// Propsito: Pone las dimensiones del cubo
//------------------------------------------------------------------------------
BOOL CPanel::Dimensiones(float rAncho, float rAlto)
{
	Ancho = rAncho;
	Alto = rAlto;

	ActualizarVertices();

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: ActualizarVertices
// Propsito: Pone el origen del cubo
//------------------------------------------------------------------------------
BOOL CPanel::Posicion(float x, float y, float z)
{
	m_rX = x;
	m_rY = y;
	m_rZ = z;

	ActualizarVertices();

	return TRUE;
}