//-------------------------------------------------------------------------------
// CCrono.cpp
//-------------------------------------------------------------------------------

#include "CCrono.h"


//-----------------------------------------------------------------------------
// Funcin: CTimer()
// Propsito: Constructor de la clase. Inicializa los miembros de la clase 
// con valores por defecto.
//-----------------------------------------------------------------------------
CCrono::CCrono()
{
	// Necesitamos saber con qu frecuencia se actualiza el reloj interno del PC
	// Probamos primero con el reloj de alta resolucin del PC 
	// Si no existe, usamos timeGetTime
    if(!QueryPerformanceFrequency((LARGE_INTEGER *)&TicksPorSegundo))
        TicksPorSegundo = 1000;

	FPS = 0;
	EnMarcha = FALSE;
}

//-----------------------------------------------------------------------------
// Funcin: Empezar()
// Propsito: Resetea el contador e inicia el cronmetro.
//-----------------------------------------------------------------------------
void CCrono::Empezar()
{
	// Tomamos el tiempo actual para saber en qu momento se
	// activ el cronmetro...probamos primero con el reloj
	// de alta resolucin del PC. Si no existe, usamos timeGetTime
    if(!QueryPerformanceCounter((LARGE_INTEGER *)&TicksBase))
    {
        TicksBase = (UINT64)timeGetTime();
    }

	EnMarcha = TRUE;

	UltimaActualizacion = 0;
	NumFrames = 0;

	TiempoFrame = 0;
	TiempoDelta = 0;
}

//-----------------------------------------------------------------------------
// Funcin: Parar()
// Propsito: Para el cronmetro.
//-----------------------------------------------------------------------------
void CCrono::Parar()
{
	if(EnMarcha)
	{
		// Recuerda cuando se ha parado para poder sacar el tiempo
		// transcurrido en detencion.
		// Probamos primero con el reloj de alta resolucin del PC 
		// Si no existe usamos timeGetTime
		if(!QueryPerformanceCounter((LARGE_INTEGER *)&TicksParado))
		{
			TicksParado = (UINT64)timeGetTime();
		}

		EnMarcha = FALSE;
	}
}

//-----------------------------------------------------------------------------
// Funcin: Continuar()
// Propsito: Iniciamos el cronmetro pero sin resetear las variables.
//-----------------------------------------------------------------------------
void CCrono::Continuar()
{
	if(!EnMarcha)
	{
		UINT64 Ticks;

		// Tomamos el tiempo actual
		if(!QueryPerformanceCounter((LARGE_INTEGER *)&Ticks))
		{
			Ticks = (UINT64)timeGetTime();
		}

		// Incrementamos TicksBase para saber 
		// cunto tiempo hemos estado pausados.
		TicksBase += Ticks - TicksParado;

		EnMarcha = TRUE;
	}
}

//-----------------------------------------------------------------------------
// Funcin: DaTiempoActual()
// Propsito: Devuelve el tiempo actual en milesimas de segundo.
//-----------------------------------------------------------------------------
float CCrono::DaTiempoActual()
{
	UINT64 Ticks;

	if(EnMarcha)
	{
		if(!QueryPerformanceCounter((LARGE_INTEGER *)&Ticks))
		{
			Ticks = (UINT64)timeGetTime();
		}
	}
	else
		Ticks = TicksParado;

	// Restamos el tiempo cuando comanzamos para
	// calcular el tiempo que nuestro cronmetro
	// ha estado en marcha
	Ticks -= TicksBase;

	// Devolvemos el tiempo en segundos.
	return (float)(__int64)Ticks/(float)(__int64)TicksPorSegundo;
}

//-----------------------------------------------------------------------------
// Funcin: Frame()
// Propsito: Calcula el frame-rate. Esta funcin debe llamarse una vez por frame.
//-----------------------------------------------------------------------------
void CCrono::Frame()
{
	TiempoDelta = DaTiempoActual() - TiempoFrame;
	TiempoFrame += TiempoDelta;

	// Aumentamos el n de frames
	NumFrames++;
	// Calculamos el frame-rate
	if(TiempoFrame - UltimaActualizacion > ACTUALIZACION_DEL_FPS)
	{
		FPS = NumFrames / (TiempoFrame - UltimaActualizacion);
		NumFrames = 0;
		UltimaActualizacion = TiempoFrame;
	}
}

//-----------------------------------------------------------------------------
// Funcin: DaFps()
// Propsito: Devuelve el frame-rate.
//-----------------------------------------------------------------------------
float CCrono::DaFps()
{
	return FPS;
}

//-----------------------------------------------------------------------------
// Funcin: DaTiempoDelFrame()
// Propsito: Devuelve el tiempo cuando Frame() fue llamado por ltima vez.
//-----------------------------------------------------------------------------
float CCrono::DaTiempoDelFrame()
{
	return TiempoFrame;
}

//-----------------------------------------------------------------------------
// Funcin: DaTiempoDelta()
// Propsito: Devuelve el tiempo que ha transcurrido entre dos llamadas a Frame().
//-----------------------------------------------------------------------------
float CCrono::DaTiempoDelta()
{
	return TiempoDelta;
}

//-----------------------------------------------------------------------------
// Funcin: EstaEnMarcha()
// Propsito: Nos dice si el cronmetro est en marcha o no.
//-----------------------------------------------------------------------------
BOOL CCrono::EstaEnMarcha()
{
	return EnMarcha;
}