//-------------------------------------------------------------------------------
// CMalla.cpp
//-------------------------------------------------------------------------------

#include "CMalla.h"

//----------------------------------------------------------------------------
// Funcin: Constructor
//----------------------------------------------------------------------------
CMalla::CMalla(LPDIRECT3DDEVICE9 pD3DDevice, LPSTR pNombreArchivo)
{
	LPD3DXBUFFER pMaterialsBuffer = NULL;
	LPD3DXMESH pMesh = NULL;	

	m_pD3DDevice = pD3DDevice;
	
	if(FAILED(D3DXLoadMeshFromX(pNombreArchivo, D3DXMESH_SYSTEMMEM, m_pD3DDevice, NULL, 
                                &pMaterialsBuffer, NULL, &m_dwNumMateriales, &pMesh)))
	{
		m_pMalla = NULL;
		m_pMallaMateriales = NULL;
		m_pMallaTexturas = NULL;

		return;
	}

    D3DXMATERIAL* matMaterials = (D3DXMATERIAL*)pMaterialsBuffer->GetBufferPointer();
    
	//Crea dos arrays, uno para los materiales y otro para las texturas
	m_pMallaMateriales = new D3DMATERIAL9[m_dwNumMateriales];
    m_pMallaTexturas  = new LPDIRECT3DTEXTURE9[m_dwNumMateriales];

    for(DWORD i = 0; i < m_dwNumMateriales; i++)
    {
        // Copia el material
        m_pMallaMateriales[i] = matMaterials[i].MatD3D;

        // Pone el color ambiental del material (D3DX no lo hace slo)
		m_pMallaMateriales[i].Ambient = m_pMallaMateriales[i].Diffuse;

		// Crea la textura
        if(FAILED(D3DXCreateTextureFromFile(m_pD3DDevice, 
                                            matMaterials[i].pTextureFilename, 
                                            &m_pMallaTexturas[i])))
        {
            m_pMallaTexturas[i] = NULL;
        }
    }

    //Hemos terminado con el material asique lo eliminamos
    BorrarInter(pMaterialsBuffer);

	
	// Nos aseguramos que las normales estan listas para nuestra malla
	pMesh->CloneMeshFVF(D3DXMESH_MANAGED, D3DFVF_VERTMALLA, m_pD3DDevice, &m_pMalla);
	BorrarInter(pMesh);

	D3DXComputeNormals(m_pMalla,NULL);

}

//----------------------------------------------------------------------------
// Funcin: Destructor
//----------------------------------------------------------------------------
CMalla::~CMalla()
{
	BorrarObj(m_pMallaMateriales);

	if(m_pMallaTexturas != NULL)
	{
		for(DWORD i = 0; i < m_dwNumMateriales; i++)
		{
			if(m_pMallaTexturas[i])
			{
				BorrarInter(m_pMallaTexturas[i]);
			}
		}
	}

	BorrarObj(m_pMallaTexturas);
	BorrarInter(m_pMalla);
    
}

//------------------------------------------------------------------------------
// Funcin: CargarEstados
// Propsito: Carga los estados necesarios para pintar la malla en condiciones
//------------------------------------------------------------------------------
void CMalla::CargarEstados(void)
{
	// Especificamos los estados de render y texture-stages que necesitamos
	m_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,			FALSE);
	m_pD3DDevice->SetRenderState(D3DRS_SRCBLEND,					D3DBLEND_SRCALPHA);
	m_pD3DDevice->SetRenderState(D3DRS_DESTBLEND,					D3DBLEND_INVSRCALPHA);
	m_pD3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE,				FALSE);
	m_pD3DDevice->SetRenderState(D3DRS_FILLMODE,					D3DFILL_SOLID);
	m_pD3DDevice->SetRenderState(D3DRS_CULLMODE,					D3DCULL_CCW);
	m_pD3DDevice->SetRenderState(D3DRS_LIGHTING,					TRUE);
	m_pD3DDevice->SetRenderState(D3DRS_ZENABLE,						TRUE);
	m_pD3DDevice->SetRenderState(D3DRS_STENCILENABLE,				FALSE);
	m_pD3DDevice->SetRenderState(D3DRS_CLIPPING,					TRUE);
	m_pD3DDevice->SetRenderState(D3DRS_CLIPPLANEENABLE,				FALSE);
	m_pD3DDevice->SetRenderState(D3DRS_VERTEXBLEND,					FALSE);
	m_pD3DDevice->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE,	FALSE);
	m_pD3DDevice->SetRenderState(D3DRS_FOGENABLE,					FALSE);

	m_pD3DDevice->SetTextureStageState(0,D3DTSS_COLORARG1,				D3DTA_TEXTURE);
	m_pD3DDevice->SetTextureStageState(0,D3DTSS_COLORARG2,				D3DTA_DIFFUSE);
	m_pD3DDevice->SetTextureStageState(0,D3DTSS_COLOROP,				D3DTOP_MODULATE);
	m_pD3DDevice->SetTextureStageState(0,D3DTSS_ALPHAARG1,				D3DTA_TEXTURE);
	m_pD3DDevice->SetTextureStageState(0,D3DTSS_ALPHAOP,				D3DTOP_MODULATE);
	m_pD3DDevice->SetTextureStageState(0,D3DTSS_ALPHAARG2,				D3DTA_DIFFUSE);
	m_pD3DDevice->SetTextureStageState(0,D3DTSS_TEXCOORDINDEX,			D3DTSS_TCI_PASSTHRU);
	m_pD3DDevice->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,	D3DTTFF_DISABLE);
	m_pD3DDevice->SetTextureStageState(1,D3DTSS_COLOROP,				D3DTOP_DISABLE);
	m_pD3DDevice->SetTextureStageState(1,D3DTSS_ALPHAOP,				D3DTOP_DISABLE);

	m_pD3DDevice->SetSamplerState(0,D3DSAMP_MINFILTER,					D3DTEXF_LINEAR);
	m_pD3DDevice->SetSamplerState(0,D3DSAMP_MAGFILTER,					D3DTEXF_LINEAR);
	m_pD3DDevice->SetSamplerState(0,D3DSAMP_MIPFILTER,					D3DTEXF_LINEAR);

}

//----------------------------------------------------------------------------
// Funcin: Renderizar
// Propsito: Renderiza la malla
//----------------------------------------------------------------------------
DWORD CMalla::Renderizar()
{
	CargarEstados();

	if(m_pMalla != NULL)
	{
		for(DWORD i = 0; i < m_dwNumMateriales; i++)
		{
			m_pD3DDevice->SetMaterial(&m_pMallaMateriales[i]);
			m_pD3DDevice->SetTexture(0, m_pMallaTexturas[i]);
        
			m_pMalla->DrawSubset(i);
		}

		return m_pMalla->GetNumFaces();
	}
	else
	{
		return 0;
	}
}
