//-------------------------------------------------------------------------------
// CFuente.cpp
//-------------------------------------------------------------------------------

#include "CFuente.h"


//------------------------------------------------------------------------------
// Funcin: Constructor
//------------------------------------------------------------------------------
CFuente::CFuente(HWND Wnd,void *DebInHandle,void *DebOutHandle)
{
	hWnd=Wnd;
	debugInputHandle=DebInHandle;
	debugOutputHandle=DebOutHandle;

	Util= new CUtil(hWnd,debugInputHandle,debugOutputHandle);

	// Valores por defecto

	Ancho=16;
	Alto=16;

	CarPorCol=16;
	CarPorFila=16;

	AnchoPant=WND_WIDTH;
	AltoPant=WND_HEIGHT;

	Color=D3DCOLOR_XRGB(255,255,255);

	Sombra=FALSE;

	IncX=0.0f;
	IncY=0.0f;

	Textura=NULL;
	DispD3D=NULL;
}

//------------------------------------------------------------------------------
// Funcin: Destructor
//------------------------------------------------------------------------------
CFuente::~CFuente()
{
	if (Textura!=NULL)
	{
		BorrarInter(Textura);
		Util->EscribirLog("-> Descargada la textura %s",Archivo);
	}

	BorrarObj(Util);
}

//------------------------------------------------------------------------------
// Funcin: Destruir
// Propsito: Descarga la textura de la fuente
//------------------------------------------------------------------------------
void CFuente::Destruir()
{
	BorrarInter(Textura);
	Util->EscribirLog("-> Descargada la textura %s",Archivo);
}

//------------------------------------------------------------------------------
// Funcin: CargarFuente
// Propsito: Carga la textura de la fuente
//------------------------------------------------------------------------------
HRESULT CFuente::CargarFuente(LPDIRECT3DDEVICE9 pD3DDev,char *szArchivo)
{
	DispD3D=pD3DDev;
	Archivo=szArchivo;

	// Cargamos la textura que ser el set de carcteres
	if(FAILED(D3DXCreateTextureFromFile(DispD3D,szArchivo,&Textura)))
	{
		Util->EscribirLog("\t*- No se pudo cargar la textura %s!!",Archivo);
		return E_FAIL;
	}

	// Tomamos informacin sobre la textura
	D3DSURFACE_DESC desc;
	Textura->GetLevelDesc(0,&desc);

	Util->EscribirLog("\t-> Cargada la textura %s",Archivo);
	
	CarPorCol=desc.Width >> 4;		// Esto es lo mismo que (desc.Width / 16)
	CarPorFila=desc.Height >> 4;	// Esto es lo mismo que (desc.Height / 16)

	CrearTablaCoord();

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: PintaTextoUV
// Propsito: Pinta el texto en coordenadas porcentuales de la pantalla
//------------------------------------------------------------------------------
HRESULT CFuente::PintaTextoUV(int iU,int iV,char *szString,...)
{
	int i=0;
	va_list msg;
	char buffer[MAX_LONG]={'\0'};

	CargarEstados();

	D3DVIEWPORT9 vista;

	DispD3D->GetViewport(&vista);

	AnchoPant = vista.Width;
	AltoPant = vista.Height;

	// Pasamos de tanto % a coordenadas de pantalla
	float x=(float)((AnchoPant * iU) / 100);
	float y=(float)((AltoPant * iV) / 100);

	float IniX=x;

	va_start(msg,szString);
	_vsnprintf(buffer,MAX_LONG-1,szString,msg);
	va_end(msg);

	if(strlen(buffer)>MAX_LONG)
		return E_FAIL;

	char *strText=&buffer[0];

	// Especificamos la textura que utilizaremos para el render de los carcteres
	DispD3D->SetFVF(D3DFVF_VERTFUENTE);
	DispD3D->SetTexture(0,Textura);
	while(*strText)
	{
		BYTE c=*strText++;

		if(c=='\n')
		{
			x=IniX;
			y+=Alto;
		}
		if(c < ' ')
			continue;

		// Render
		if(FAILED(PintaLetra(x,y,c)))
			return E_FAIL;

		x+=Ancho;
	}

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: PintaTextoXY
// Propsito: Pinta texto en cordenadas X Y
//------------------------------------------------------------------------------
HRESULT CFuente::PintaTextoXY(int iX,int iY,char *szString,...)
{
	int i=0;
	va_list msg;
	char buffer[MAX_LONG]={'\0'};

	CargarEstados();

	D3DVIEWPORT9 vista;

	DispD3D->GetViewport(&vista);

	AnchoPant = vista.Width;
	AltoPant = vista.Height;

	float x=(float)iX;
	float y=(float)iY;

	float IniX=x;

	va_start(msg,szString);
	_vsnprintf(buffer,MAX_LONG-1,szString,msg);
	va_end(msg);

	if(strlen(buffer)>MAX_LONG)
		return E_FAIL;

	char *strText=&buffer[0];

	// Especificamos la textura que utilizaremos para el render de los carcteres
	DispD3D->SetFVF(D3DFVF_VERTFUENTE);
	DispD3D->SetTexture(0,Textura);
	while(*strText)
	{
		BYTE c=*strText++;

		if(c=='\n')
		{
			x=IniX;
			y+=Alto;
		}
		if(c < ' ')
			continue;

		// Render
		if(FAILED(PintaLetra(x,y,c)))
			return E_FAIL;

		x+=Ancho;
	}

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: PintaLineaXY
// Propsito: Pinta una linea centrada, a la izquierda o a la derecha
// NOTA: Si se escribe el caracter de retorno (\n) ste se ignorar
//		 ya que esta funcin esta pensada para trabajar con una sla linea.	
//------------------------------------------------------------------------------
HRESULT CFuente::PintaLineaXY(int iX,int iY,int iAlineacion,char *szString,...)
{
	int i=0;
	va_list msg;
	char buffer[MAX_LONG]={'\0'};

	CargarEstados();

	D3DVIEWPORT9 vista;

	DispD3D->GetViewport(&vista);

	AnchoPant = vista.Width;
	AltoPant = vista.Height;

	va_start(msg,szString);
	_vsnprintf(buffer,MAX_LONG-1,szString,msg);
	va_end(msg);

	// Pasamos a coordenadas alineadas
	
	float x;
	x=(float)iX;

	switch (iAlineacion)
	{
		case TEXTO_IZQUIERDA:

			break;

		case TEXTO_DERECHA:
			x-=Ancho*strlen(buffer);
			break;

		case TEXTO_CENTRO:
			x-=(Ancho*strlen(buffer))/2;
			break;
	}

	
	float y=(float)iY;

	float IniX=x;


	if(strlen(buffer)>MAX_LONG)
		return E_FAIL;

	char *strText=&buffer[0];

	// Especificamos la textura que utilizaremos para el render de los carcteres
	DispD3D->SetFVF(D3DFVF_VERTFUENTE);
	DispD3D->SetTexture(0,Textura);
	while(*strText)
	{
		BYTE c=*strText++;

		if(c < ' ')
			continue;

		// Render
		if(FAILED(PintaLetra(x,y,c)))
			return E_FAIL;

		x+=Ancho;
	}

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: CargarEstados
// Propsito: Carga los estados necesarios para pintar el texto en condiciones
//------------------------------------------------------------------------------
void CFuente::CargarEstados(void)
{
	// Especificamos los estados de render y texture-stages que necesitamos
	DispD3D->SetRenderState(D3DRS_ALPHABLENDENABLE,			TRUE);
	DispD3D->SetRenderState(D3DRS_SRCBLEND,					D3DBLEND_SRCALPHA);
	DispD3D->SetRenderState(D3DRS_DESTBLEND,				D3DBLEND_INVSRCALPHA);
	DispD3D->SetRenderState(D3DRS_ALPHATESTENABLE,			FALSE);
	DispD3D->SetRenderState(D3DRS_FILLMODE,					D3DFILL_SOLID);
	DispD3D->SetRenderState(D3DRS_CULLMODE,					D3DCULL_CCW);
	DispD3D->SetRenderState(D3DRS_LIGHTING,					FALSE);
	DispD3D->SetRenderState(D3DRS_ZENABLE,					FALSE);
	DispD3D->SetRenderState(D3DRS_STENCILENABLE,			FALSE);
	DispD3D->SetRenderState(D3DRS_CLIPPING,					TRUE);
	DispD3D->SetRenderState(D3DRS_CLIPPLANEENABLE,			FALSE);
	DispD3D->SetRenderState(D3DRS_VERTEXBLEND,				FALSE);
	DispD3D->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE,	FALSE);
	DispD3D->SetRenderState(D3DRS_FOGENABLE,				FALSE);

	DispD3D->SetTextureStageState(0,D3DTSS_COLORARG1,				D3DTA_TEXTURE);
	DispD3D->SetTextureStageState(0,D3DTSS_COLORARG2,				D3DTA_DIFFUSE);
	DispD3D->SetTextureStageState(0,D3DTSS_COLOROP,					D3DTOP_MODULATE);
	DispD3D->SetTextureStageState(0,D3DTSS_ALPHAARG1,				D3DTA_TEXTURE);
	DispD3D->SetTextureStageState(0,D3DTSS_ALPHAOP,					D3DTOP_MODULATE);
	DispD3D->SetTextureStageState(0,D3DTSS_ALPHAARG2,				D3DTA_DIFFUSE);
	DispD3D->SetTextureStageState(0,D3DTSS_TEXCOORDINDEX,			D3DTSS_TCI_PASSTHRU);
	DispD3D->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,	D3DTTFF_DISABLE);
	DispD3D->SetTextureStageState(1,D3DTSS_COLOROP,					D3DTOP_DISABLE);
	DispD3D->SetTextureStageState(1,D3DTSS_ALPHAOP,					D3DTOP_DISABLE);

	DispD3D->SetSamplerState(0,D3DSAMP_MINFILTER,					D3DTEXF_LINEAR);
	DispD3D->SetSamplerState(0,D3DSAMP_MAGFILTER,					D3DTEXF_LINEAR);
	DispD3D->SetSamplerState(0,D3DSAMP_MIPFILTER,					D3DTEXF_LINEAR);
}

//------------------------------------------------------------------------------
// Funcin: PintaLetra
//------------------------------------------------------------------------------
HRESULT CFuente::PintaLetra(float fX,float fY,BYTE c)
{
	VERTFUENTE vertices[4];

	// Primero la sombra si la hay
	if(Sombra)
	{
		float fOffset = 2.0f;

		vertices[0].vertice=D3DXVECTOR4(fX+fOffset,fY+Alto+fOffset,1.0f,1.0f);
		vertices[0].color=D3DCOLOR_XRGB(0,0,0);
		vertices[0].s=CoordTexto[c][0];
		vertices[0].t=CoordTexto[c][1]+IncY;

		vertices[1].vertice=D3DXVECTOR4(fX+fOffset,fY+fOffset,1.0f,1.0f);
		vertices[1].color=D3DCOLOR_XRGB(0,0,0);
		vertices[1].s=CoordTexto[c][0];
		vertices[1].t=CoordTexto[c][1];

		vertices[2].vertice=D3DXVECTOR4(fX+Ancho+fOffset,fY+Alto+fOffset,1.0f,1.0f);
		vertices[2].color=D3DCOLOR_XRGB(0,0,0);
		vertices[2].s=CoordTexto[c][0]+IncX;
		vertices[2].t=CoordTexto[c][1]+IncY;

		vertices[3].vertice=D3DXVECTOR4(fX+Ancho+fOffset,fY+fOffset,1.0f,1.0f);
		vertices[3].color=D3DCOLOR_XRGB(0,0,0);
		vertices[3].s=CoordTexto[c][0]+IncX;
		vertices[3].t=CoordTexto[c][1];

		if(FAILED(DispD3D->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vertices,
				sizeof(VERTFUENTE))))
			return E_FAIL;
	}

	// Ahora el carcter en s
	vertices[0].vertice = D3DXVECTOR4(fX, fY + Alto, 1.0f, 1.0f);
	vertices[0].color = Color;
	vertices[0].s = CoordTexto[c][0];
	vertices[0].t = CoordTexto[c][1] + IncY;

	vertices[1].vertice = D3DXVECTOR4(fX, fY, 1.0f, 1.0f);
	vertices[1].color = Color;
	vertices[1].s = CoordTexto[c][0];
	vertices[1].t = CoordTexto[c][1];

	vertices[2].vertice = D3DXVECTOR4(fX + Ancho, fY + Alto, 1.0f, 1.0f);
	vertices[2].color = Color;
	vertices[2].s = CoordTexto[c][0] + IncX;
	vertices[2].t = CoordTexto[c][1] + IncY;

	vertices[3].vertice = D3DXVECTOR4(fX+Ancho,fY,1.0f,1.0f);
	vertices[3].color = Color;
	vertices[3].s = CoordTexto[c][0]+IncX;
	vertices[3].t = CoordTexto[c][1];

	if(FAILED(DispD3D->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vertices,
			sizeof(VERTFUENTE))))
		return E_FAIL;

	return S_OK;
}

///------------------------------------------------------------------------------
// Funcin: Dimensiones
// Propsito: Pone las dimensiones del caracter
//------------------------------------------------------------------------------
void CFuente::Dimensiones(int iAncho, int iAlto)
{
	Ancho = iAncho;
	Alto = iAlto;
}

//------------------------------------------------------------------------------
// Funcin: Color
// Propsito: Pone el color (puede tener canal Alpha para texto transparente)
//------------------------------------------------------------------------------
void CFuente::PonerColor(DWORD dwColor)
{
	Color = dwColor;
}

//------------------------------------------------------------------------------
// Funcin: Sombra
// Propsito: Pone o quita la sombra
//------------------------------------------------------------------------------
void CFuente::PonerSombra(BOOL bSombra)
{
	Sombra = bSombra;
}

//------------------------------------------------------------------------------
// Funcin: CrearTablaCoord
//------------------------------------------------------------------------------
void CFuente::CrearTablaCoord(void)
{
#define vCopy(d,x,y) { d[0]=x;d[1]=y; }

	int i=0;
	float x,y;
	
	IncX=1.0f / (float)CarPorCol;
	IncY=1.0f / (float)CarPorFila;

	for(y=0.0f;y<1.0f;y+=IncY)
	{
		for(x=0.0f;x<1.0f;x+=IncX,i++)
		{
			vCopy(CoordTexto[i],x,y);
		}
	}


#undef vCopy
}