//-------------------------------------------------------------------------------
// CInput.cpp
//-------------------------------------------------------------------------------

#include "CInput.h"


//------------------------------------------------------------------------------
// Funcin: CInput
// Propsito: Constructor. Inicia los miembros de la clase
//------------------------------------------------------------------------------

CInput::CInput()
{
	ObjDinput = NULL;
	Teclado = NULL;
	Raton = NULL;
	EstIzqdo=EstDcho=EstCentral=0;

	PULSADO_IZQUIERDO=FALSE;
	PULSADO_DERECHO=FALSE;
	PULSADO_CENTRAL=FALSE;

	SOLTADO_IZQUIERDO=TRUE;
	SOLTADO_DERECHO=TRUE;
	SOLTADO_CENTRAL=TRUE;

}

//------------------------------------------------------------------------------
// Funcin: ~CInput
// Propsito: Destructor. Elimina los miembros de la clase necesarios
//------------------------------------------------------------------------------

CInput::~CInput()
{
	if(Teclado)
	{
		Teclado->Unacquire();
		Teclado->Release();
		Teclado = NULL;
	}

	if(Raton)
	{
		Raton->Unacquire();
		Raton->Release();
		Raton = NULL;
	}

	if(ObjDinput)
	{
		ObjDinput->Release();
		ObjDinput = NULL;
	}
}

//------------------------------------------------------------------------------
// Funcin: Inicializar
// Propsito: Inicializa DInput, realiza la creacin del teclado y del ratn.
//------------------------------------------------------------------------------

HRESULT CInput::Inicializar(HWND hWnd,BOOL RatonExclusivo,BOOL TecladoExclusivo)
{
	/////////////////////////////////////////////////////////////////////////////////////
	// Creamos el objeto DInput
	if(FAILED(DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION, IID_IDirectInput8, (VOID**)&ObjDinput, NULL)))
	{
		MessageBox(hWnd, "No se pudo inicializar DInput!", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}
	/////////////////////////////////////////////////////////////////////////////////////

	
	/////////////////////////////////////////////////////////////////////////////////////
	// Creamos el device Keyboard
	if(FAILED(ObjDinput->CreateDevice(GUID_SysKeyboard, &Teclado, NULL)))
	{
		MessageBox(hWnd, "No se pudo crear el objeto teclado!", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	// Especificamos el formato de los datos
	if(FAILED(Teclado->SetDataFormat(&c_dfDIKeyboard)))
	{
		MessageBox(hWnd, "No se pudo especificar el formato de datos del teclado!", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	// Especificamos el modo cooperativo
	if(TecladoExclusivo)
	{
		if(FAILED(Teclado->SetCooperativeLevel(hWnd, DISCL_EXCLUSIVE|DISCL_FOREGROUND)))
		{
			MessageBox(hWnd, "No se pudo especificar el formato de datos del teclado!", "Error", MB_OK|MB_ICONEXCLAMATION);
			return E_FAIL;
		}
	}
	else
	{
		if(FAILED(Teclado->SetCooperativeLevel(hWnd, DISCL_NONEXCLUSIVE|DISCL_FOREGROUND)))
		{
			MessageBox(hWnd, "No se pudo especificar el formato de datos del teclado!", "Error", MB_OK|MB_ICONEXCLAMATION);
			return E_FAIL;
		}
	}
	

	// Adquirimos device keyboard
	Teclado->Acquire();
	/////////////////////////////////////////////////////////////////////////////////////


	/////////////////////////////////////////////////////////////////////////////////////
	// Creamos el device Mouse
	if(FAILED(ObjDinput->CreateDevice(GUID_SysMouse, &Raton, NULL)))
	{
		MessageBox(hWnd, "No se pudo crear el objeto mouse!", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	// Especificamos el formato de los datos
	if(FAILED(Raton->SetDataFormat(&c_dfDIMouse2)))
	{
		MessageBox(hWnd, "No se pudo especificar el formato de datos del mouse!", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	// Especificamos el modo cooperativo
	if (RatonExclusivo)
	{
		if(FAILED(Raton->SetCooperativeLevel(hWnd, DISCL_EXCLUSIVE |DISCL_FOREGROUND)))
		{
			MessageBox(hWnd, "No se pudo especificar el formato de datos del mouse!", "Error", MB_OK|MB_ICONEXCLAMATION);
			return E_FAIL;
		}
	}
	else
	{
		if(FAILED(Raton->SetCooperativeLevel(hWnd, DISCL_NONEXCLUSIVE |DISCL_FOREGROUND)))
		{
			MessageBox(hWnd, "No se pudo especificar el formato de datos del mouse!", "Error", MB_OK|MB_ICONEXCLAMATION);
			return E_FAIL;
		}
	}
	

	// Adquirimos device mouse
	Raton->Acquire();
	/////////////////////////////////////////////////////////////////////////////////////

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: Actualizar
// Propsito: Actualiza el teclado y el ratn en cada FRAME.
//------------------------------------------------------------------------------

HRESULT CInput::Actualizar(void)
{
	if(FAILED(EstadoDelTeclado()))
	{
		return E_FAIL;
	}

	if(FAILED(EstadoDelRaton()))
	{
		return E_FAIL;
	}

	//////////////////////////////////////

	if (EstIzqdo==0)
	{
		if (BotonPulsado(BOTON_IZQUIERDO))
		{
			PULSADO_IZQUIERDO=FALSE;
			SOLTADO_IZQUIERDO=FALSE;
			EstIzqdo=1;
		}
		else
		{
			PULSADO_IZQUIERDO=FALSE;
			SOLTADO_IZQUIERDO=TRUE;
		}
	}
	else
	{
		if (!BotonPulsado(BOTON_IZQUIERDO))
		{
			EstIzqdo=0;
			PULSADO_IZQUIERDO=TRUE;
			SOLTADO_IZQUIERDO=TRUE;
		}
	}

	//////////////////////////////////////

	if (EstDcho==0)
	{
		if (BotonPulsado(BOTON_DERECHO))
		{
			PULSADO_DERECHO=FALSE;
			SOLTADO_DERECHO=FALSE;
			EstDcho=1;
		}
		else
		{
			PULSADO_DERECHO=FALSE;
			SOLTADO_DERECHO=TRUE;
		}
	}
	else
	{
		if (!BotonPulsado(BOTON_DERECHO))
		{
			EstDcho=0;
			PULSADO_DERECHO=TRUE;
			SOLTADO_DERECHO=TRUE;
		}
	}

	///////////////////////////////////////

	if (EstCentral==0)
	{
		if (BotonPulsado(BOTON_CENTRAL))
		{
			PULSADO_CENTRAL=FALSE;
			SOLTADO_CENTRAL=FALSE;
			EstCentral=1;
		}
		else
		{
			PULSADO_CENTRAL=FALSE;
			SOLTADO_CENTRAL=TRUE;
		}
	}
	else
	{
		if (!BotonPulsado(BOTON_CENTRAL))
		{
			EstCentral=0;
			PULSADO_CENTRAL=TRUE;
			SOLTADO_CENTRAL=TRUE;
		}
	}

	///////////////////////////////////////

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: TeclaPulsada
// Propsito: Dice si una tecla ha sido pulsada o no.
//------------------------------------------------------------------------------
BOOL CInput::TeclaPulsada(int iKey)
{
	// Nos aseguramos de que iKey sea un entero vlido
	if(iKey < 0 || iKey > 255)
		return FALSE;

	// Miramos si la tecla iKey ha sido pulsada
	if(Teclas[iKey] & 0x80)
		return TRUE;

	return FALSE;
}

//------------------------------------------------------------------------------
// Funcin: BotonPulsado
// Propsito: Dice si un botn del ratn ha sido pulsado o no.
//------------------------------------------------------------------------------
BOOL CInput::BotonPulsado(int iButton)
{
	if(EstadoRaton.rgbButtons[iButton] & 0x80)
		return TRUE;
	else return FALSE;
}

//------------------------------------------------------------------------------
// Funcin: XAbsolutaRaton
// Propsito: Devuelve la coordenada X absoluta (en coordenadas de pantalla) del
// ratn.
//------------------------------------------------------------------------------
int CInput::XAbsolutaRaton(void)
{
	static int iBasePos;

	return iBasePos += EstadoRaton.lX;
}

//------------------------------------------------------------------------------
// Funcin: YAbsolutaRaton
// Propsito: Devuelve la coordenada Y absoluta (en coordenadas de pantalla) del
// ratn.
//------------------------------------------------------------------------------
int CInput::YAbsolutaRaton(void)
{
	static int iBasePos;

	return iBasePos += EstadoRaton.lY;
}

//------------------------------------------------------------------------------
// Funcin: ZAbsolutaRaton
// Propsito: Devuelve la coordenada Z absoluta (en coordenadas de pantalla) del
// ratn.
//------------------------------------------------------------------------------
int CInput::ZAbsolutaRaton(void)
{
	static int iBasePos;

	return iBasePos += EstadoRaton.lZ;
}

//------------------------------------------------------------------------------
// Funcin: XRelativaRaton
// Propsito: Devuelve la coordenada X relativa del ratn.
//------------------------------------------------------------------------------
int CInput::XRelativaRaton(void)
{
	return EstadoRaton.lX;
}

//------------------------------------------------------------------------------
// Funcin: YRelativaRaton
// Propsito: Devuelve la coordenada Y relativa del ratn.
//------------------------------------------------------------------------------
int CInput::YRelativaRaton(void)
{
	return EstadoRaton.lY;
}

//------------------------------------------------------------------------------
// Funcin: ZRelativaRaton
// Propsito: Devuelve la coordenada Z relativa del ratn.
//------------------------------------------------------------------------------
int CInput::ZRelativaRaton(void)
{
	return EstadoRaton.lZ;
}

//------------------------------------------------------------------------------
// Funcin: EstadoDelTeclado
// Propsito: Actualiza el teclado rellenando el array de teclas.
//------------------------------------------------------------------------------
HRESULT CInput::EstadoDelTeclado(void)
{
	HRESULT hr;

	if(!Teclado)
		return S_OK;
    
	// Recogemos el estado del keyboard y rellenamos el buffer de teclas
	ZeroMemory(&Teclas, sizeof(Teclas));
	if(FAILED(Teclado->GetDeviceState(sizeof(Teclas), &Teclas))) 
	{
		// Si hemos perdido el device entonces intentamos recuperarlo
		hr = Teclado->Acquire();
		while(hr == DIERR_INPUTLOST)
			hr = Teclado->Acquire();

		return S_OK; 
	}

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: EstadoDelRaton
// Propsito: Actualiza el ratn rellenando la estructura de estado.
//------------------------------------------------------------------------------
HRESULT CInput::EstadoDelRaton(void)
{
	HRESULT hr;

	if(!Raton)
		return S_OK;
    
	// Recogemos el estado del ratn y rellenamos la estructura de estado
	if(FAILED(Raton->GetDeviceState(sizeof(DIMOUSESTATE2), &EstadoRaton))) 
	{
		// Si hemos perdido el device entonces intentamos recuperarlo
		hr = Raton->Acquire();
		while(hr == DIERR_INPUTLOST)
			hr = Raton->Acquire();

		return S_OK; 
	}

	return S_OK;
}