//-------------------------------------------------------------------------------
// CNivelQ3.cpp
//-------------------------------------------------------------------------------

#include "CNivelQ3.h"

//------------------------------------------------------------------------------
// Funcin: Constructor
//------------------------------------------------------------------------------
CNivelQ3::CNivelQ3(LPDIRECT3DDEVICE9 pDispD3D)
{
	m_iNumDeVerts    = 0;	
	m_iNumDePolis    = 0;	
	m_iNumDeTexturas = 0;

	m_pD3DDev=pDispD3D;

	m_pVerts=NULL;
	m_pPolis=NULL;

	m_pBufferVert=NULL;
	m_pTextura=NULL;
}

//------------------------------------------------------------------------------
// Funcin: Destructor
//------------------------------------------------------------------------------
CNivelQ3::~CNivelQ3()
{
	Finalizar();
}

//------------------------------------------------------------------------------
// Funcin: CargarEstados
// Propsito: Carga los estados necesarios para pintar el nivel en condiciones
//------------------------------------------------------------------------------
void CNivelQ3::CargarEstados(void)
{
	// Especificamos los estados de render y texture-stages que necesitamos
	m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE,			FALSE);
	m_pD3DDev->SetRenderState(D3DRS_SRCBLEND,					D3DBLEND_SRCALPHA);
	m_pD3DDev->SetRenderState(D3DRS_DESTBLEND,					D3DBLEND_ONE);
	m_pD3DDev->SetRenderState(D3DRS_ALPHATESTENABLE,			FALSE);
	m_pD3DDev->SetRenderState(D3DRS_FILLMODE,					GFX_OPC_RELLENO);
	m_pD3DDev->SetRenderState(D3DRS_CULLMODE,					D3DCULL_CCW);
	m_pD3DDev->SetRenderState(D3DRS_LIGHTING,					FALSE);
	m_pD3DDev->SetRenderState(D3DRS_ZENABLE,					TRUE);
	m_pD3DDev->SetRenderState(D3DRS_STENCILENABLE,				TRUE);
	m_pD3DDev->SetRenderState(D3DRS_CLIPPING,					TRUE);
	m_pD3DDev->SetRenderState(D3DRS_CLIPPLANEENABLE,			FALSE);
	m_pD3DDev->SetRenderState(D3DRS_VERTEXBLEND,				FALSE);
	m_pD3DDev->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE,	FALSE);
	m_pD3DDev->SetRenderState(D3DRS_FOGENABLE,					FALSE);
	
	m_pD3DDev->SetTextureStageState(0,D3DTSS_COLOROP,				D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_COLORARG1,				D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_COLORARG2,				D3DTA_DIFFUSE);
		
	m_pD3DDev->SetTextureStageState(0,D3DTSS_ALPHAARG1,				D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_ALPHAARG2,				D3DTA_DIFFUSE);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_ALPHAOP,				D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_TEXCOORDINDEX,			D3DTSS_TCI_PASSTHRU);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,	D3DTTFF_DISABLE);

	m_pD3DDev->SetSamplerState(0,D3DSAMP_MINFILTER,					D3DTEXF_LINEAR);
	m_pD3DDev->SetSamplerState(0,D3DSAMP_MAGFILTER,					D3DTEXF_LINEAR);
	m_pD3DDev->SetSamplerState(0,D3DSAMP_MIPFILTER,					D3DTEXF_LINEAR);
	m_pD3DDev->SetSamplerState(0,D3DSAMP_ADDRESSU,					D3DTADDRESS_WRAP);
	m_pD3DDev->SetSamplerState(0,D3DSAMP_ADDRESSV,					D3DTADDRESS_WRAP);
}

//------------------------------------------------------------------------------
// Funcin: BuscarExtTextura
// Propsito: Busca la extensin de una textura
//------------------------------------------------------------------------------
HRESULT CNivelQ3::BuscarExtTextura(char *strArchivo)
{
	if(strlen(strArchivo)>65)
		return E_FAIL;

	char strJPGPath[TEXTO_LONG_MAX] = {0};
	char strTGAPath[TEXTO_LONG_MAX]    = {0}; 
	FILE *fp = NULL;

	//////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////

	// Directorio actual
	GetCurrentDirectory(MAX_PATH, strJPGPath);

	// Adimos la barra "\"
	strcat(strJPGPath, "\\");
	strcat(strJPGPath, strArchivo);
	strcpy(strTGAPath, strJPGPath);
	
	// Aadimos la extension
	strcat(strJPGPath, ".jpg");
	strcat(strTGAPath, ".tga");

	//////////////////////////////////////////////////////////////////////

	// Vemos si existe el archivo con extension jpg
	if((fp = fopen(strJPGPath, "rb")) != NULL)
	{
		// Si existe le aadimos la extension y voila!!
		strcat(strArchivo, ".jpg");
		return S_OK;
	}

	//////////////////////////////////////////////////////////////////////

	// Vemos si existe el archivo con extension jpg
	if((fp = fopen(strTGAPath, "rb")) != NULL)
	{
		// Si existe le aadimos la extension y voila!!
		strcat(strArchivo, ".tga");
		return S_OK;
	}

	// Sino, pues debe ser una textura especial o dada en el archivo shader,
	// o una textura base del Quake.

	return E_FAIL;
}

//------------------------------------------------------------------------------
// Funcin: CrearBufferVert
// Propsito: Crea el buffer de vrtices 
//------------------------------------------------------------------------------
HRESULT CNivelQ3::CrearBufferVert(int NumVertices)
{

	if(FAILED(m_pD3DDev->CreateVertexBuffer(NumVertices * sizeof(VERTICE),
											D3DUSAGE_WRITEONLY,
											D3DFVF_VERTICE,
											D3DPOOL_MANAGED,
											&m_pBufferVert,
											NULL)))
											
	{
        return E_FAIL;
    }

    return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: CargarBSP
// Propsito: Carga el archivo
//------------------------------------------------------------------------------
HRESULT CNivelQ3::CargarBSP(const char *strNombreArchivo)
{
	FILE *fp = NULL;
	int i = 0;

	//////////////////////////////////////////////////////////////////////
	
	// Miramos si existe el archivo
	if((fp = fopen(strNombreArchivo, "rb")) == NULL)
	{
		return E_FAIL;
	}

	//////////////////////////////////////////////////////////////////////

	// Inicializamos la cabecera y los bloques
	tBSPHeader header = {0};
	tBSPBloque bloques[kMaxBloques] = {0};

	// Leemos la cabecera y los bloques
	fread(&header, 1, sizeof(tBSPHeader), fp);

	fread(&bloques, kMaxBloques, sizeof(tBSPBloque), fp);

	//////////////////////////////////////////////////////////////////////

	tBSPTextura *textura;
	
	// Reservamos memoria para el array de vertices
	m_iNumDeVerts = bloques[kVertices].longitud / sizeof(tBSPVertice);
	m_pVerts = new tBSPVertice [m_iNumDeVerts];
	
	// Reservamos memoria para el array de poligonos
	m_iNumDePolis = bloques[kPolis].longitud / sizeof(tBSPPoli);
	m_pPolis     = new tBSPPoli [m_iNumDePolis];

	// Reservamos memoria para el array de texturas
	m_iNumDeTexturas = bloques[kTexturas].longitud / sizeof(tBSPTextura);
	textura = new tBSPTextura [m_iNumDeTexturas];

	/////////////////////////////////////////////////////////////////////

	// Buscamos las estructuras de los poligonos en el archivo
	fseek(fp, bloques[kPolis].desplaz, SEEK_SET);

	// Leemos los polis
	fread(m_pPolis, m_iNumDePolis, sizeof(tBSPPoli), fp);

	/////////////////////////////////////////////////////////////////////

	// Buscamos las estructuras de los vertices en el archivo
	fseek(fp, bloques[kVertices].desplaz, SEEK_SET);

	// Leemos los vertices
	fread(m_pVerts, m_iNumDeVerts, sizeof(tBSPVertice), fp);

	if(FAILED(CrearBufferVert(m_iNumDeVerts)))
	{
		return E_FAIL;
	}

	if(!ActualizarVertices())
	{
		return E_FAIL;
	}
	
	/////////////////////////////////////////////////////////////////////

	// Buscamos las estructuras de las texturas en el archivo
	fseek(fp, bloques[kTexturas].desplaz, SEEK_SET);
	
	// Leemos las texturas
	fread(textura, m_iNumDeTexturas, sizeof(tBSPTextura), fp);

	// Ya tenemos los datos de las texturas, ahora las cargamos:

	// Creamos el array y lo incializamos a null
	if (m_iNumDeTexturas)
		m_pTextura = new LPDIRECT3DTEXTURE9 [m_iNumDeTexturas];
	for (i=0;i<m_iNumDeTexturas;i++)
	{
		m_pTextura[i]=NULL;
	}

	// Pasamos por todas las texturas
	for(i = 0; i < m_iNumDeTexturas; i++)
	{
		// Busca la extension y la aade al nombre
		if (SUCCEEDED(BuscarExtTextura(textura[i].strNombre)))
		{
			// crea la textura a partir del archivo
			CargarTextura(textura[i].strNombre,i);
		}
	}

	// podemos borrar el array temporal de estructuras de textura
	BorrarObj(textura);

	//////////////////////////////////////////////////////////////////////

	// Creamos la tabla de bits de poligonos pintados
	m_PolisPintados.Inicializar(m_iNumDePolis);

	// Cerramos el archivo
	fclose(fp);

	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: ActualizarVertices
// Propsito: Actualiza los cambios en los vrtices
//------------------------------------------------------------------------------
BOOL CNivelQ3::ActualizarVertices(void)
{
  
    VERTICE *Vertice;
	
	if(FAILED(m_pBufferVert->Lock(0, 0, (void**)&Vertice, 0)))
    {
        return FALSE;
    }

	for (int i=0; i<m_iNumDeVerts; i++)
	{
		
		Vertice[i].pos.x=m_pVerts[i].vPosicion.x;
		Vertice[i].pos.y=m_pVerts[i].vPosicion.z;
		Vertice[i].pos.z=m_pVerts[i].vPosicion.y;
		
		Vertice[i].normal.x=m_pVerts[i].vNormal.x;
		Vertice[i].normal.y=m_pVerts[i].vNormal.z;
		Vertice[i].normal.z=m_pVerts[i].vNormal.y;

		Vertice[i].u=m_pVerts[i].vTexturaCoord.x;
		Vertice[i].v=m_pVerts[i].vTexturaCoord.y;

		Vertice[i].lu=m_pVerts[i].vLightmapCoord.x;
		Vertice[i].lv=m_pVerts[i].vLightmapCoord.y;

	}

    m_pBufferVert->Unlock();

	return TRUE;
}

//------------------------------------------------------------------------------
// Funcin: CargarTextura
// Propsito: Carga la textura de un archivo
//------------------------------------------------------------------------------
HRESULT CNivelQ3::CargarTextura(const char *szArchivo, int indice)
{


	if(FAILED(D3DXCreateTextureFromFileEx(m_pD3DDev,
										  szArchivo,
										  D3DX_DEFAULT,
										  D3DX_DEFAULT,
										  10,
										  D3DUSAGE_RENDERTARGET,
										  D3DFMT_UNKNOWN,
										  D3DPOOL_DEFAULT,
										  D3DX_DEFAULT,
										  D3DX_DEFAULT,
										  0,
										  NULL,
										  NULL,
										  &m_pTextura[indice])))
	{
		return E_FAIL;
	}
	
	return S_OK;
}

//------------------------------------------------------------------------------
// Funcin: RenderizarPoli
// Propsito: Renderiza un polgono en triangle fan
//------------------------------------------------------------------------------
void CNivelQ3::RenderizarPoli(int iNumDePoli)
{
	//creamos esto para manejarnos mejor
	//dentro del array de polis cogemos el que tenga nuestro indice iNumDePoli
	tBSPPoli *pPoli = &m_pPolis[iNumDePoli];
	
	//si existe el array de texturas ponemos la que le corresponede
	if (m_pTextura != NULL)
	{
		m_pD3DDev->SetTexture(0, m_pTextura[pPoli->texturaID]);
	}
	
	// lo pintamos
	m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLEFAN, pPoli->startVertIndex, pPoli->numDeVerts-2);

	// acutulizamos contadores
	m_iVertsPintados += pPoli->numDeVerts;
	m_iPolisPintados ++;	

}

//------------------------------------------------------------------------------
// Funcin: RenderizarNivel
// Propsito: Renderiza el nivel
//------------------------------------------------------------------------------
void CNivelQ3::RenderizarNivel(const D3DXVECTOR3 &vPos)
{

	m_PolisPintados.DesactivarTodos(); 

	m_iVertsPintados=0;
	m_iPolisPintados=0;

	//Ponemos la matriz identidad en WORLD por si algo la habia cambiado
	D3DXMATRIX Tras;
	D3DXMatrixIdentity(&Tras);
	m_pD3DDev->SetTransform(D3DTS_WORLD, &Tras);

	// elegimos nuestro tipo de vrtice y cargamos estados
	m_pD3DDev->SetStreamSource(0, m_pBufferVert, 0, sizeof(VERTICE));
    m_pD3DDev->SetFVF(D3DFVF_VERTICE);

	CargarEstados();

	// recorremos todos los polis y los pintamos uno a uno
	// si son poligonos solidos normales y no se han pintado ya
	for (int i=0; i<m_iNumDePolis;i++)
	{
		// Aseguramos que es un poli normal
		if(m_pPolis[i].tipo != POLIGONO_SOLIDO) continue;

		// Si el poli no se ha dibujado ya...
		if(!m_PolisPintados.EstaActivado(i)) 
		{
			// Pintamos el poli y lo activamos como pintado
			RenderizarPoli(i);
			m_PolisPintados.Activar(i);
			
		}
	}
}


//------------------------------------------------------------------------------
// Funcin: Finalizar
// Propsito: Finaliza el objeto
//------------------------------------------------------------------------------
void CNivelQ3::Finalizar(void)
{
	BorrarObj(m_pVerts);
	BorrarObj(m_pPolis);

	// Borramos el buffer de vertices
	BorrarInter(m_pBufferVert);

	// Borramos las texturas
	if(m_pTextura) 
	{
		
		for (int i=0;i<m_iNumDeTexturas;i++)
		{
			BorrarInter(m_pTextura[i]);
		}

		BorrarObj(m_pTextura);
	}
}