//-------------------------------------------------------------------------------
// CNivelQ3.h
//-------------------------------------------------------------------------------

#ifndef CNIVELQ3_H
#define CNIVELQ3_H

#include <d3dx9.h>
#include "principal.h"
#include "CBitset.h"

#define POLIGONO_SOLIDO		1
#define POLIGONO_PATCH		2
#define POLIGONO_MALLA		3
#define POLIGONO_BILLBOARD	4

#define D3DFVF_VERTICE (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1|D3DFVF_TEX2)

//////////////////////////////////////////////////////////////////////
typedef struct sVERTICE
{
	D3DXVECTOR3 pos;
	D3DXVECTOR3 normal;

	float u,v;
	float lu,lv;

} VERTICE;
//////////////////////////////////////////////////////////////////////
typedef struct sVECTOR3BSP
{
	float x, y, z;
	
} VECTOR3BSP;
//////////////////////////////////////////////////////////////////////
typedef struct sVECTOR2BSP
{
	float x, y;
	
} VECTOR2BSP;
//////////////////////////////////////////////////////////////////////
typedef struct sVECTOR3BSPi
{
	int x, y, z;				
}VECTOR3BSPi;
//////////////////////////////////////////////////////////////////////
struct tBSPHeader
{
    char strID[4];				// Siempre deber ser 'IBSP'
    int version;				// Debe ser 0x2e para archivos del Quake 3
}; 
//////////////////////////////////////////////////////////////////////
struct tBSPBloque
{
	int desplaz;				// El desplazamiento en el archivo para el comienzo de este bloque
	int longitud;				// Longitud en bytes del bloque
};
//////////////////////////////////////////////////////////////////////
struct tBSPVertice
{
    VECTOR3BSP vPosicion;		// Posicion (x, y, z) 
    VECTOR2BSP vTexturaCoord;	// Coordenadas de textura (u, v)
    VECTOR2BSP vLightmapCoord;	// Coordenadas de lightmap (u, v)
    VECTOR3BSP vNormal;			// Vector normal (x, y, z)
    byte color[4];				// color RGBA para el vrtice 
};
//////////////////////////////////////////////////////////////////////
struct tBSPPoli
{
    int texturaID;				// El indice en el array de texturas 
    int efecto;					// El indice para los efectos (-1 significa desactivado)
    int tipo;					// 1=poligono, 2=superficie curva, 3=malla, 4=billboard 
    int startVertIndex;			// El primer inidice para el primer vertice del poligono
    int numDeVerts;				// Numero de vertices del poli 
    int meshVertIndex;			// El inidice en el primer vertice de malla
    int numMeshVerts;			// El numero de vertices en la malla 
    int lightmapID;				// El indice de textura para el lightmap 
    int lMapCorner[2];			// La esquina del lightmap en la imagen 
    int lMapSize[2];			// El tamo de la seccin del lightmap
    VECTOR3BSP lMapPos;			// El punto de origen 3D del lightmap. 
    VECTOR3BSP lMapVecs[2];		// Coordenadas de los vectores unitarios s y t 
    VECTOR3BSP vNormal;			// La normal del polgono 
    int size[2];				// Los tamaos de la superficie curva 
};
//////////////////////////////////////////////////////////////////////
struct tBSPTextura
{
    char strNombre[64];			// Nombre de la textura sin la extension 
    int flags;					// Flags de la superficie 
    int contenidos;				// Contenido de los flags (desconocido)
};
//////////////////////////////////////////////////////////////////////
struct tBSPLightmap
{
	byte mapaDeBits[128][128][3]; // Datos RGB en una imagen de 128x128  
};
//////////////////////////////////////////////////////////////////////
enum eBloquess
{
    kEntidades = 0,				// Almacena posicion de jugadores, objetos, etc
    kTexturas,					// Almacena informacion de texturas
    kPlanos,				    // Almacena los planos de particion
    kNodos,						// Almacena los nodos BSP
    kHojas,						// Almacena las hojas de los nodos
    kPoliHojas,					// Almacena los indices de las hojas en las caras
    kVoluHojas,					// Almacena los indices de las hojas en los volumenes
    kModelos,					// Almacena la informacion de los modelos
    kVolumenes,					// Almacena la informacin de los volmenes (para colisin)
    kVoluLados,					// Almacena la informacin de las superficies del volumen
    kVertices,					// Almacena los verices del nivel
    kVertMalla,					// Almacena los deplazamientos de los vertices de modelos
    kShaders,					// Almacena los archivos de shader
    kPolis,						// Almacena los poligonos del nivel
    kLightmaps,					// Almacena los lightmaps del nivel
    kLightVolumes,				// Almacena informacion extra sobre iluminacion
    kVisData,					// Almacena el PVS (Potential Visibility Set) para el culling
    kMaxBloques					// Una constante para saber el numero de bloques
};

//-------------------------------------------------------------------------------
// Clase: CNivelQ3
//-------------------------------------------------------------------------------

class CNivelQ3 {

	private:

		LPDIRECT3DDEVICE9			m_pD3DDev;
		LPDIRECT3DVERTEXBUFFER9		m_pBufferVert;
		LPDIRECT3DTEXTURE9			*m_pTextura; // Array de texturas
		LPDIRECT3DTEXTURE9			*m_pLightmap; // Array de lightmaps

		tBSPVertice  *m_pVerts;	
		tBSPPoli *m_pPolis;		
		
		CBitset m_PolisPintados;

		HRESULT BuscarExtTextura(char *strArchivo);
		HRESULT CargarTextura(const char *szArchivo, int indice);
		void RenderizarPoli(int iNumDePoli);
		HRESULT CrearBufferVert(int NumVertices);
		BOOL ActualizarVertices(void);
		void CargarEstados(void);
		HRESULT CrearTexturaLightmap(int indice, byte pMapaDeBits[128][128][3], int ancho, int alto);
		void CambiarGamma(byte *pImagen, int tamanno, float factor);

		
	public:

		int  m_iNumDeVerts;
		int  m_iNumDePolis;
		int  m_iNumDeTexturas;
		int  m_iNumDeLightmaps;
		int	 m_iVertsPintados;
		int  m_iPolisPintados;
		
		CNivelQ3(LPDIRECT3DDEVICE9 pDispD3D);

		~CNivelQ3(void);

		HRESULT CargarBSP(const char *strNombreArchivo);
		void RenderizarNivel(const D3DXVECTOR3 &vPos);
		void Finalizar(void);

		
};

#endif // CNIVELQ3_H