//-------------------------------------------------------------------------------
// Tutorial del Win32
// PLANET ISO
// http://www.planetiso.cjb.net
//-------------------------------------------------------------------------------

#include <windows.h>

//-------------------------------------------------------------------------------
// Definiciones
//-------------------------------------------------------------------------------
#define APP_NAME	"Prueba de Win32"
#define WND_WIDTH	640
#define WND_HEIGHT	480

// Nos dice si una tecla determinada ha sido pulsada
#define GET_KEYDOWN(key) ((int)((GetAsyncKeyState(key) & (1 << 15)) >> 15))

//-------------------------------------------------------------------------------
// Declaracin de funciones
//-------------------------------------------------------------------------------
LRESULT WINAPI MsgProc(HWND, UINT, WPARAM, LPARAM);
INT WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int);

HRESULT	IniVentana(HINSTANCE hInstance);

//-----------------------------------------------------------------------------
// Declaracin de variabes globales
//-----------------------------------------------------------------------------
HWND	g_hWnd;				// Nuestra ventana

//----------------------------------------------------------------------------
// Funcin: IniVentana
// Propsito: Inicializa nuestra ventana
//----------------------------------------------------------------------------
HRESULT IniVentana(HINSTANCE hInstance)
{
	
	WNDCLASSEX	wc;		// clase de ventana
	
	// Rellenamos la estructura de clase de ventana
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_CLASSDC;
	wc.lpfnWndProc = MsgProc;
	wc.cbClsExtra = 0L;
	wc.cbWndExtra = 0L;
	wc.hInstance = GetModuleHandle(NULL);
	wc.hIcon = NULL;
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = APP_NAME;
	wc.hIconSm = NULL;


	// Registramos la clase de vetana
	if(!RegisterClassEx(&wc))
	{
		MessageBox(NULL, "No se pudo registrar la clase de ventana", "Error", MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
	}

	// Creamos la ventana
	g_hWnd = CreateWindow(APP_NAME,				// nombre de la clase
						APP_NAME,				// ttulo de la ventana
						WS_OVERLAPPEDWINDOW,	// tipo de ventana
						CW_USEDEFAULT,			// posicin X de la ventana
						CW_USEDEFAULT,			// posicin Y de a ventana
						WND_WIDTH,				// ancho de la ventana
						WND_HEIGHT,				// alto de la ventana
						GetDesktopWindow(),		// ventana padre
						NULL,					// men
						wc.hInstance,			// instancia de la aplicacin
						NULL);					// no se suele usar

	if (!g_hWnd)
    {
        MessageBox(NULL,"No se cre el objeto g_hWnd","Error",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;
    }

	return S_OK;
}

//-------------------------------------------------------------------------------
// Funcin: MsgProc
// Propsito: Manejar los mensajes de la aplicacin
//-------------------------------------------------------------------------------
LRESULT WINAPI MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;

		case WM_PAINT:

			ValidateRect(hWnd, NULL);
			return 0;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

//----------------------------------------------------------------------------
// Funcin: WinMain
// Propsito; Es la funcin principal de toda aplicacin de Windows
// Se encarga de crear la ventana, mostrarla y es la receptora de los
// mensajes que le son enviados a la aplicacin
//----------------------------------------------------------------------------
INT WINAPI WinMain(HINSTANCE hInstance,			// Instancia a nuestra aplicacin
				   HINSTANCE hPrevInstance,		// Existen instancias previas de nuestra aplicacin?
				   LPSTR lpCmdLine,				// Lnea de comandos
				   int nShowCmd)				// Cmo se despliega nuestra ventana?
{
	
	if(!SUCCEEDED(IniVentana(hInstance)))
		return 0;

	
	// Mostrar nuestra ventana
	ShowWindow(g_hWnd, nShowCmd);
	UpdateWindow(g_hWnd);
	
	// Bucle de mensajes tpico de Windows
	MSG	msg;	// handle a un mensaje
	ZeroMemory(&msg, sizeof(msg));

	while(msg.message != WM_QUIT)
	{
		// si se pulsa ESC salimos
		if(GET_KEYDOWN(VK_ESCAPE))
			PostQuitMessage(0);

		if(PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			//Nuestra funcin de render aqu
		}
	}
	
    // Salir de la aplicacin
	
	UnregisterClass(APP_NAME,hInstance);

	return (0);
}